package kr.co.vividnext.sodalive.audio_content.all.by_theme

import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import com.orhanobut.logger.Logger
import io.reactivex.rxjava3.android.schedulers.AndroidSchedulers
import io.reactivex.rxjava3.schedulers.Schedulers
import kr.co.vividnext.sodalive.audio_content.AudioContentRepository
import kr.co.vividnext.sodalive.audio_content.AudioContentViewModel
import kr.co.vividnext.sodalive.base.BaseViewModel
import kr.co.vividnext.sodalive.common.SharedPreferenceManager

class AudioContentAllByThemeViewModel(
    private val repository: AudioContentRepository
) : BaseViewModel() {
    private val _toastLiveData = MutableLiveData<String?>()
    val toastLiveData: LiveData<String?>
        get() = _toastLiveData

    private var _isLoading = MutableLiveData(false)
    val isLoading: LiveData<Boolean>
        get() = _isLoading

    private var _contentListLiveData = MutableLiveData<GetContentByThemeResponse>()
    val contentListLiveData: LiveData<GetContentByThemeResponse>
        get() = _contentListLiveData

    private val _sort = MutableLiveData(AudioContentViewModel.Sort.NEWEST)
    val sort: LiveData<AudioContentViewModel.Sort>
        get() = _sort

    private var isLast = false
    var page = 1
    private val size = 15

    fun getContentList(themeId: Long) {
        if (!_isLoading.value!! && !isLast) {
            _isLoading.value = true

            compositeDisposable.add(
                repository.getAudioContentByTheme(
                    themeId = themeId,
                    page = page,
                    size = size,
                    sort = _sort.value!!,
                    token = "Bearer ${SharedPreferenceManager.token}"
                )
                    .subscribeOn(Schedulers.io())
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe(
                        {
                            if (it.success && it.data != null) {
                                if (it.data.items.isNotEmpty()) {
                                    page += 1
                                    _contentListLiveData.postValue(it.data!!)
                                } else {
                                    isLast = true
                                }
                            } else {
                                if (it.message != null) {
                                    _toastLiveData.postValue(it.message)
                                } else {
                                    _toastLiveData.postValue(
                                        "알 수 없는 오류가 발생했습니다. 다시 시도해 주세요."
                                    )
                                }
                            }

                            _isLoading.value = false
                        },
                        {
                            _isLoading.value = false
                            it.message?.let { message -> Logger.e(message) }
                            _toastLiveData.postValue("알 수 없는 오류가 발생했습니다. 다시 시도해 주세요.")
                        }
                    )
            )
        }
    }

    fun changeSort(sort: AudioContentViewModel.Sort) {
        page = 1
        isLast = false
        _sort.postValue(sort)
    }
}
