package kr.co.vividnext.sodalive.audio_content.main.v2

import android.content.Intent
import android.graphics.Typeface
import android.view.ViewGroup
import android.widget.TextView
import androidx.core.content.res.ResourcesCompat
import com.google.android.material.tabs.TabLayout
import kr.co.vividnext.sodalive.R
import kr.co.vividnext.sodalive.audio_content.box.AudioContentBoxActivity
import kr.co.vividnext.sodalive.audio_content.main.v2.alarm.AudioContentMainTabAlarmFragment
import kr.co.vividnext.sodalive.audio_content.main.v2.asmr.AudioContentMainTabAsmrFragment
import kr.co.vividnext.sodalive.audio_content.main.v2.content.AudioContentMainTabContentFragment
import kr.co.vividnext.sodalive.audio_content.main.v2.free.AudioContentMainTabFreeFragment
import kr.co.vividnext.sodalive.audio_content.main.v2.replay.AudioContentMainTabReplayFragment
import kr.co.vividnext.sodalive.audio_content.main.v2.series.AudioContentMainTabSeriesFragment
import kr.co.vividnext.sodalive.base.BaseActivity
import kr.co.vividnext.sodalive.common.Constants
import kr.co.vividnext.sodalive.databinding.ActivityAudioContentMainBinding
import kr.co.vividnext.sodalive.mypage.alarm.AlarmListActivity
import kotlin.math.min

enum class AudioContentMainTab {
    HOME, SERIES, CONTENT, ALARM, ASMR, REPLAY, FREE;

    companion object {
        fun fromOrdinal(ordinal: Int): AudioContentMainTab? {
            return values().getOrNull(ordinal)
        }
    }
}

class AudioContentMainActivity : BaseActivity<ActivityAudioContentMainBinding>(
    ActivityAudioContentMainBinding::inflate
) {

    private var fontBold: Typeface? = null
    private var fontMedium: Typeface? = null

    private var startTabPosition: AudioContentMainTab = AudioContentMainTab.SERIES

    override fun setupView() {
        startTabPosition = AudioContentMainTab.fromOrdinal(
            intent.getIntExtra(
                Constants.EXTRA_START_TAB_POSITION,
                AudioContentMainTab.SERIES.ordinal
            )
        ) ?: AudioContentMainTab.SERIES

        setupToolbar()
        loadFont()
        setupTabs()
    }

    private fun setupToolbar() {
        val toolbar = binding.toolbar

        toolbar.ivContentKeep.setOnClickListener {
            startActivity(
                Intent(
                    applicationContext,
                    AudioContentBoxActivity::class.java
                )
            )
        }

        toolbar.ivAlarm.setOnClickListener {
            startActivity(
                Intent(
                    applicationContext,
                    AlarmListActivity::class.java
                )
            )
        }
    }

    private fun loadFont() {
        fontBold = ResourcesCompat.getFont(this, R.font.gmarket_sans_bold)
        fontMedium = ResourcesCompat.getFont(this, R.font.gmarket_sans_medium)
    }

    private fun setupTabs() {
        val tabs = binding.tabs
        val tabTitles = listOf("홈", "시리즈", "단편", "모닝콜", "ASMR", "다시듣기", "무료")
        for (title in tabTitles) {
            tabs.addTab(tabs.newTab().setText(title))
        }

        tabs.addOnTabSelectedListener(object : TabLayout.OnTabSelectedListener {
            override fun onTabSelected(tab: TabLayout.Tab) {
                val selectedTab = AudioContentMainTab.fromOrdinal(tab.position)
                if (selectedTab == null || selectedTab == AudioContentMainTab.HOME) finish()

                replaceFragment(selectedTab = selectedTab!!)
                tab.view.isSelected = true
                setTabFont(tab, fontBold)
            }

            override fun onTabUnselected(tab: TabLayout.Tab) {
                tab.view.isSelected = false
                setTabFont(tab, fontMedium)
            }

            override fun onTabReselected(tab: TabLayout.Tab) {
            }
        })

        tabs.getTabAt(startTabPosition.ordinal)?.let {
            it.select()
            scrollToTab(tabs, startTabPosition.ordinal)
        }
        replaceFragment(selectedTab = startTabPosition)
    }

    private fun scrollToTab(tabLayout: TabLayout, position: Int) {
        tabLayout.post {
            val layout = tabLayout.getChildAt(0) as ViewGroup
            val tabView = layout.getChildAt(position)

            // 화면 전체 너비
            val parentWidth = tabLayout.width
            // 선택한 탭의 중심 좌표
            val tabCenterX = tabView.left + tabView.width / 2
            // 스크롤 할 위치 = 탭의 중심을 화면 중앙에 배치
            val scrollToX = tabCenterX - parentWidth / 2

            tabLayout.scrollTo(min(tabView.left, scrollToX), 0)
        }
    }

    private fun replaceFragment(selectedTab: AudioContentMainTab) {
        val startFragment = when (selectedTab) {
            AudioContentMainTab.CONTENT -> AudioContentMainTabContentFragment()
            AudioContentMainTab.ALARM -> AudioContentMainTabAlarmFragment()
            AudioContentMainTab.ASMR -> AudioContentMainTabAsmrFragment()
            AudioContentMainTab.REPLAY -> AudioContentMainTabReplayFragment()
            AudioContentMainTab.FREE -> AudioContentMainTabFreeFragment()
            else -> AudioContentMainTabSeriesFragment()
        }

        supportFragmentManager.beginTransaction()
            .replace(
                R.id.fl_container,
                startFragment
            )
            .commit()
    }

    private fun setTabFont(tab: TabLayout.Tab, font: Typeface?) {
        (tab.view.getChildAt(1) as? TextView)?.typeface = font
    }
}
