package kr.co.vividnext.sodalive.audio_content.main.v2

import android.annotation.SuppressLint
import android.content.Context
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.core.content.ContextCompat
import androidx.recyclerview.widget.RecyclerView
import coil.load
import coil.transform.CircleCropTransformation
import kr.co.vividnext.sodalive.R
import kr.co.vividnext.sodalive.audio_content.main.ContentCreatorResponse
import kr.co.vividnext.sodalive.databinding.ItemContentRankCreatorBinding

class ContentRankCreatorAdapter(
    private val onClickItem: (Long) -> Unit,
) : RecyclerView.Adapter<ContentRankCreatorAdapter.ViewHolder>() {

    private var selectedCreatorId: Long = 0
    private val items = mutableListOf<ContentCreatorResponse>()

    inner class ViewHolder(
        private val context: Context,
        private val binding: ItemContentRankCreatorBinding
    ) : RecyclerView.ViewHolder(binding.root) {
        @SuppressLint("NotifyDataSetChanged")
        fun bind(item: ContentCreatorResponse) {
            binding.root.setOnClickListener {
                if (selectedCreatorId != item.creatorId) {
                    selectedCreatorId = item.creatorId
                    onClickItem(item.creatorId)
                    notifyDataSetChanged()
                }
            }

            binding.tvNickname.text = item.creatorNickname
            binding.ivProfile.load(item.creatorProfileImageUrl) {
                transformations(CircleCropTransformation())
                placeholder(R.drawable.ic_place_holder)
                crossfade(true)
            }

            if (item.creatorId == selectedCreatorId) {
                binding.ivBg.setImageResource(R.drawable.bg_circle_3bb9f1)
                binding.ivBg.visibility = View.VISIBLE
                binding.tvNickname.setTextColor(
                    ContextCompat.getColor(
                        context,
                        R.color.color_3bb9f1
                    )
                )
            } else {
                binding.ivBg.setImageResource(0)
                binding.ivBg.visibility = View.GONE
                binding.tvNickname.setTextColor(
                    ContextCompat.getColor(
                        context,
                        R.color.color_bbbbbb
                    )
                )
            }
        }
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int) = ViewHolder(
        parent.context,
        ItemContentRankCreatorBinding.inflate(
            LayoutInflater.from(parent.context),
            parent,
            false
        )
    )

    override fun onBindViewHolder(holder: ViewHolder, position: Int) {
        holder.bind(items[position])
    }

    override fun getItemCount() = items.size

    @SuppressLint("NotifyDataSetChanged")
    fun addItems(items: List<ContentCreatorResponse>) {
        this.items.addAll(items)
        if (this.items.isNotEmpty()) {
            this.selectedCreatorId = this.items[0].creatorId
        }
        notifyDataSetChanged()
    }
}
