package kr.co.vividnext.sodalive.audio_content.main.v2.alarm.all

import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import com.orhanobut.logger.Logger
import io.reactivex.rxjava3.android.schedulers.AndroidSchedulers
import io.reactivex.rxjava3.schedulers.Schedulers
import kr.co.vividnext.sodalive.audio_content.main.GetAudioContentMainItem
import kr.co.vividnext.sodalive.audio_content.main.v2.alarm.AudioContentMainTabAlarmRepository
import kr.co.vividnext.sodalive.base.BaseViewModel
import kr.co.vividnext.sodalive.common.SharedPreferenceManager

class AlarmContentAllViewModel(
    private val repository: AudioContentMainTabAlarmRepository
) : BaseViewModel() {
    private val _toastLiveData = MutableLiveData<String?>()
    val toastLiveData: LiveData<String?>
        get() = _toastLiveData

    private var _isLoading = MutableLiveData(false)
    val isLoading: LiveData<Boolean>
        get() = _isLoading

    private var _newContentListLiveData = MutableLiveData<List<GetAudioContentMainItem>>()
    val newContentListLiveData: LiveData<List<GetAudioContentMainItem>>
        get() = _newContentListLiveData

    private var isLast = false
    private var page = 1
    private val size = 10
    private var selectedTheme = ""

    fun getContentMainAlarmAll() {
        if (!_isLoading.value!! && !isLast) {
            _isLoading.value = true

            compositeDisposable.add(
                repository.getContentMainAlarmAll(
                    theme = if (selectedTheme == "전체") {
                        ""
                    } else {
                        selectedTheme
                    },
                    page = page,
                    size = size,
                    token = "Bearer ${SharedPreferenceManager.token}"
                )
                    .subscribeOn(Schedulers.io())
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe(
                        {
                            if (it.success && it.data != null) {
                                if (it.data.isNotEmpty()) {
                                    page += 1
                                    _newContentListLiveData.value = it.data!!
                                } else {
                                    isLast = true
                                }
                            } else {
                                if (it.message != null) {
                                    _toastLiveData.postValue(it.message)
                                } else {
                                    _toastLiveData.postValue(
                                        "알 수 없는 오류가 발생했습니다. 다시 시도해 주세요."
                                    )
                                }
                            }

                            _isLoading.value = false
                        },
                        {
                            _isLoading.value = false
                            it.message?.let { message -> Logger.e(message) }
                            _toastLiveData.postValue("알 수 없는 오류가 발생했습니다. 다시 시도해 주세요.")
                        }
                    )
            )
        }
    }

    fun selectTheme(theme: String) {
        isLast = false
        page = 1
        selectedTheme = theme
        getContentMainAlarmAll()
    }
}
