package kr.co.vividnext.sodalive.audio_content.main.v2.asmr

import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import com.orhanobut.logger.Logger
import io.reactivex.rxjava3.android.schedulers.AndroidSchedulers
import io.reactivex.rxjava3.schedulers.Schedulers
import kr.co.vividnext.sodalive.audio_content.main.GetAudioContentBannerResponse
import kr.co.vividnext.sodalive.audio_content.main.GetAudioContentMainItem
import kr.co.vividnext.sodalive.audio_content.main.GetAudioContentRankingItem
import kr.co.vividnext.sodalive.audio_content.main.v2.GetContentCurationResponse
import kr.co.vividnext.sodalive.base.BaseViewModel
import kr.co.vividnext.sodalive.common.SharedPreferenceManager
import kr.co.vividnext.sodalive.settings.event.EventItem

class AudioContentMainTabAsmrViewModel(
    private val repository: AudioContentMainTabAsmrRepository
) : BaseViewModel() {
    private val _toastLiveData = MutableLiveData<String?>()
    val toastLiveData: LiveData<String?>
        get() = _toastLiveData

    private var _isLoading = MutableLiveData(false)
    val isLoading: LiveData<Boolean>
        get() = _isLoading

    private var _contentBannerLiveData = MutableLiveData<List<GetAudioContentBannerResponse>>()
    val contentBannerLiveData: LiveData<List<GetAudioContentBannerResponse>>
        get() = _contentBannerLiveData

    private var _newContentListLiveData = MutableLiveData<List<GetAudioContentMainItem>>()
    val newContentListLiveData: LiveData<List<GetAudioContentMainItem>>
        get() = _newContentListLiveData

    private var _contentRankingLiveData = MutableLiveData<List<GetAudioContentRankingItem>>()
    val contentRankingLiveData: LiveData<List<GetAudioContentRankingItem>>
        get() = _contentRankingLiveData

    private val _eventLiveData = MutableLiveData<List<EventItem>>()
    val eventLiveData: LiveData<List<EventItem>>
        get() = _eventLiveData

    private var _curationListLiveData = MutableLiveData<List<GetContentCurationResponse>>()
    val curationListLiveData: LiveData<List<GetContentCurationResponse>>
        get() = _curationListLiveData

    fun fetchData() {
        _isLoading.value = true
        compositeDisposable.add(
            repository.getContentMainAsmr(token = "Bearer ${SharedPreferenceManager.token}")
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(
                    {
                        if (it.success && it.data != null) {
                            val data = it.data

                            _contentBannerLiveData.value = data.contentBannerList
                            _newContentListLiveData.value = data.newAsmrContentList
                            _contentRankingLiveData.value = data.rankAsmrContentList
                            _eventLiveData.value = data.eventBannerList.eventList
                            _curationListLiveData.value = data.curationList
                        } else {
                            if (it.message != null) {
                                _toastLiveData.postValue(it.message)
                            } else {
                                _toastLiveData.postValue(
                                    "알 수 없는 오류가 발생했습니다. 다시 시도해 주세요."
                                )
                            }
                        }

                        _isLoading.value = false
                    },
                    {
                        _isLoading.value = false
                        it.message?.let { message -> Logger.e(message) }
                        _toastLiveData.postValue("알 수 없는 오류가 발생했습니다. 다시 시도해 주세요.")
                    }
                )
        )
    }
}
