package kr.co.vividnext.sodalive.audio_content.main.v2.content

import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import com.orhanobut.logger.Logger
import io.reactivex.rxjava3.android.schedulers.AndroidSchedulers
import io.reactivex.rxjava3.schedulers.Schedulers
import kr.co.vividnext.sodalive.audio_content.main.ContentCreatorResponse
import kr.co.vividnext.sodalive.audio_content.main.GetAudioContentBannerResponse
import kr.co.vividnext.sodalive.audio_content.main.GetAudioContentMainItem
import kr.co.vividnext.sodalive.audio_content.main.GetAudioContentRankingItem
import kr.co.vividnext.sodalive.base.BaseViewModel
import kr.co.vividnext.sodalive.common.SharedPreferenceManager
import kr.co.vividnext.sodalive.settings.event.EventItem

class AudioContentMainTabContentViewModel(
    private val repository: AudioContentMainTabContentRepository
) : BaseViewModel() {
    private val _toastLiveData = MutableLiveData<String?>()
    val toastLiveData: LiveData<String?>
        get() = _toastLiveData

    private var _isLoading = MutableLiveData(false)
    val isLoading: LiveData<Boolean>
        get() = _isLoading

    private var _contentBannerLiveData = MutableLiveData<List<GetAudioContentBannerResponse>>()
    val contentBannerLiveData: LiveData<List<GetAudioContentBannerResponse>>
        get() = _contentBannerLiveData

    private var _newContentListLiveData = MutableLiveData<List<GetAudioContentMainItem>>()
    val newContentListLiveData: LiveData<List<GetAudioContentMainItem>>
        get() = _newContentListLiveData

    private var _themeListLiveData = MutableLiveData<List<String>>()
    val themeListLiveData: LiveData<List<String>>
        get() = _themeListLiveData

    private var _contentRankingSortListLiveData = MutableLiveData<List<String>>()
    val contentRankingSortListLiveData: LiveData<List<String>>
        get() = _contentRankingSortListLiveData

    private var _contentRankingLiveData = MutableLiveData<List<GetAudioContentRankingItem>>()
    val contentRankingLiveData: LiveData<List<GetAudioContentRankingItem>>
        get() = _contentRankingLiveData

    private val _eventLiveData = MutableLiveData<List<EventItem>>()
    val eventLiveData: LiveData<List<EventItem>>
        get() = _eventLiveData

    private val _contentRankCreatorListLiveData = MutableLiveData<List<ContentCreatorResponse>>()
    val contentRankCreatorListLiveData: LiveData<List<ContentCreatorResponse>>
        get() = _contentRankCreatorListLiveData

    private val _salesRankContentListLiveData = MutableLiveData<List<GetAudioContentRankingItem>>()
    val salesRankContentListLiveData: LiveData<List<GetAudioContentRankingItem>>
        get() = _salesRankContentListLiveData

    private val _salesCountRankContentListLiveData =
        MutableLiveData<List<GetAudioContentRankingItem>>()
    val salesCountRankContentListLiveData: LiveData<List<GetAudioContentRankingItem>>
        get() = _salesCountRankContentListLiveData

    fun fetchData() {
        _isLoading.value = true
        compositeDisposable.add(
            repository.getContentMainContent(token = "Bearer ${SharedPreferenceManager.token}")
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(
                    {
                        if (it.success && it.data != null) {
                            val data = it.data

                            _contentBannerLiveData.value = data.bannerList

                            val themeList = listOf("전체").union(data.contentThemeList).toList()
                            _themeListLiveData.value = themeList
                            _newContentListLiveData.value = data.newContentList

                            _contentRankingSortListLiveData.value = data.rankSortTypeList
                            _contentRankingLiveData.value = data.rankContentList

                            _eventLiveData.value = data.eventBannerList.eventList
                            _contentRankCreatorListLiveData.value = data.contentRankCreatorList
                            _salesRankContentListLiveData.value = data.salesRankContentList
                            _salesCountRankContentListLiveData.value =
                                data.salesCountRankContentList
                        } else {
                            if (it.message != null) {
                                _toastLiveData.postValue(it.message)
                            } else {
                                _toastLiveData.postValue(
                                    "알 수 없는 오류가 발생했습니다. 다시 시도해 주세요."
                                )
                            }
                        }

                        _isLoading.value = false
                    },
                    {
                        _isLoading.value = false
                        it.message?.let { message -> Logger.e(message) }
                        _toastLiveData.postValue("알 수 없는 오류가 발생했습니다. 다시 시도해 주세요.")
                    }
                )
        )
    }

    fun getNewContentOfTheme(theme: String) {
        _isLoading.value = true
        compositeDisposable.add(
            repository.getNewContentOfTheme(
                theme = if (theme == "전체") {
                    ""
                } else {
                    theme
                },
                token = "Bearer ${SharedPreferenceManager.token}"
            )
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(
                    {
                        if (it.success && it.data != null) {
                            _newContentListLiveData.value = it.data!!
                        } else {
                            if (it.message != null) {
                                _toastLiveData.postValue(it.message)
                            } else {
                                _toastLiveData.postValue(
                                    "알 수 없는 오류가 발생했습니다. 다시 시도해 주세요."
                                )
                            }
                        }

                        _isLoading.value = false
                    },
                    {
                        it.message?.let { message -> Logger.e(message) }
                        _toastLiveData.postValue("알 수 없는 오류가 발생했습니다. 다시 시도해 주세요.")
                        _isLoading.value = false
                    }
                )
        )
    }

    fun getContentRanking(sort: String = "매출") {
        _isLoading.value = true
        compositeDisposable.add(
            repository.getContentRanking(
                sortType = sort,
                token = "Bearer ${SharedPreferenceManager.token}"
            )
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(
                    {
                        if (it.success && it.data != null) {
                            _contentRankingLiveData.value = it.data!!
                        } else {
                            if (it.message != null) {
                                _toastLiveData.postValue(it.message)
                            } else {
                                _toastLiveData.postValue(
                                    "알 수 없는 오류가 발생했습니다. 다시 시도해 주세요."
                                )
                            }
                        }
                        _isLoading.value = false
                    },
                    {
                        it.message?.let { message -> Logger.e(message) }
                        _toastLiveData.postValue("알 수 없는 오류가 발생했습니다. 다시 시도해 주세요.")
                        _isLoading.value = false
                    }
                )
        )
    }

    fun getPopularContentByCreator(creatorId: Long) {
        _isLoading.value = true
        compositeDisposable.add(
            repository.getPopularContentByCreator(
                creatorId = creatorId,
                token = "Bearer ${SharedPreferenceManager.token}"
            )
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(
                    {
                        _isLoading.value = false
                        if (it.success && it.data != null) {
                            val data = it.data
                            _salesRankContentListLiveData.value = data.salesRankContentList
                            _salesCountRankContentListLiveData.value =
                                data.salesCountRankContentList
                        } else {
                            if (it.message != null) {
                                _toastLiveData.postValue(it.message)
                            } else {
                                _toastLiveData.postValue(
                                    "알 수 없는 오류가 발생했습니다. 다시 시도해 주세요."
                                )
                            }
                        }
                    },
                    {
                        _isLoading.value = false
                        it.message?.let { message -> Logger.e(message) }
                        _toastLiveData.postValue("알 수 없는 오류가 발생했습니다. 다시 시도해 주세요.")
                    }
                )
        )
    }
}
