package kr.co.vividnext.sodalive.audio_content.main.v2.free

import android.content.Intent
import android.graphics.Rect
import android.net.Uri
import android.os.Bundle
import android.view.View
import android.widget.LinearLayout
import android.widget.Toast
import androidx.annotation.OptIn
import androidx.core.content.ContextCompat
import androidx.media3.common.util.UnstableApi
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.zhpan.bannerview.BaseBannerAdapter
import com.zhpan.indicator.enums.IndicatorSlideMode
import com.zhpan.indicator.enums.IndicatorStyle
import kr.co.vividnext.sodalive.R
import kr.co.vividnext.sodalive.audio_content.all.AudioContentNewAllActivity
import kr.co.vividnext.sodalive.audio_content.detail.AudioContentDetailActivity
import kr.co.vividnext.sodalive.audio_content.main.AudioContentBannerType
import kr.co.vividnext.sodalive.audio_content.main.AudioContentMainContentAdapter
import kr.co.vividnext.sodalive.audio_content.main.banner.AudioContentMainBannerAdapter
import kr.co.vividnext.sodalive.audio_content.main.new_content.AudioContentMainNewContentThemeAdapter
import kr.co.vividnext.sodalive.audio_content.main.v2.AudioContentMainContentCurationAdapter
import kr.co.vividnext.sodalive.audio_content.main.v2.free.introduce_creator.IntroduceCreatorActivity
import kr.co.vividnext.sodalive.audio_content.main.v2.series.new_series.AudioContentMainNewSeriesAdapter
import kr.co.vividnext.sodalive.audio_content.series.detail.SeriesDetailActivity
import kr.co.vividnext.sodalive.base.BaseFragment
import kr.co.vividnext.sodalive.common.Constants
import kr.co.vividnext.sodalive.common.LoadingDialog
import kr.co.vividnext.sodalive.databinding.FragmentAudioContentMainTabFreeBinding
import kr.co.vividnext.sodalive.explorer.profile.UserProfileActivity
import kr.co.vividnext.sodalive.extensions.dpToPx
import kr.co.vividnext.sodalive.settings.event.EventDetailActivity
import org.koin.android.ext.android.inject
import kotlin.math.roundToInt

@OptIn(UnstableApi::class)
class AudioContentMainTabFreeFragment : BaseFragment<FragmentAudioContentMainTabFreeBinding>(
    FragmentAudioContentMainTabFreeBinding::inflate
) {
    private val viewModel: AudioContentMainTabFreeViewModel by inject()

    private lateinit var loadingDialog: LoadingDialog
    private lateinit var contentBannerAdapter: AudioContentMainBannerAdapter
    private lateinit var introduceCreatorAdapter: AudioContentMainContentAdapter
    private lateinit var recommendSeriesAdapter: AudioContentMainNewSeriesAdapter
    private lateinit var newContentThemeAdapter: AudioContentMainNewContentThemeAdapter
    private lateinit var newContentAdapter: AudioContentMainContentAdapter
    private lateinit var curationAdapter: AudioContentMainContentCurationAdapter

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        setupView()
        bindData()

        viewModel.fetchData()
    }

    private fun setupView() {
        loadingDialog = LoadingDialog(requireActivity(), layoutInflater)

        setupContentBanner()
        setupIntroduceCreator()
        setupRecommendSeries()
        setupNewContentTheme()
        setupNewContent()
        setupCuration()
    }

    private fun setupContentBanner() {
        val layoutParams = binding
            .rvBanner
            .layoutParams as LinearLayout.LayoutParams

        val pagerWidth = screenWidth.toDouble() - 26.7f.dpToPx()
        val pagerHeight = (pagerWidth * 0.53).roundToInt()
        layoutParams.width = pagerWidth.roundToInt()
        layoutParams.height = pagerHeight

        contentBannerAdapter = AudioContentMainBannerAdapter(
            requireContext(),
            pagerWidth.roundToInt(),
            pagerHeight
        ) {
            when (it.type) {
                AudioContentBannerType.EVENT -> {
                    startActivity(
                        Intent(requireContext(), EventDetailActivity::class.java).apply {
                            putExtra(Constants.EXTRA_EVENT, it.eventItem!!)
                        }
                    )
                }

                AudioContentBannerType.CREATOR -> {
                    startActivity(
                        Intent(requireContext(), UserProfileActivity::class.java).apply {
                            putExtra(Constants.EXTRA_USER_ID, it.creatorId!!)
                        }
                    )
                }

                AudioContentBannerType.SERIES -> {
                    startActivity(
                        Intent(requireContext(), SeriesDetailActivity::class.java).apply {
                            putExtra(Constants.EXTRA_SERIES_ID, it.seriesId!!)
                        }
                    )
                }

                AudioContentBannerType.LINK -> {
                    startActivity(Intent(Intent.ACTION_VIEW, Uri.parse(it.link!!)))
                }
            }
        }

        binding
            .rvBanner
            .layoutParams = layoutParams

        binding.rvBanner.apply {
            adapter = contentBannerAdapter as BaseBannerAdapter<Any>

            setLifecycleRegistry(lifecycle)
            setScrollDuration(1000)
            setInterval(4 * 1000)
        }.create()

        binding
            .rvBanner
            .setIndicatorView(binding.indicatorBanner)
            .setIndicatorStyle(IndicatorStyle.ROUND_RECT)
            .setIndicatorSlideMode(IndicatorSlideMode.SMOOTH)
            .setIndicatorVisibility(View.GONE)
            .setIndicatorSliderColor(
                ContextCompat.getColor(requireContext(), R.color.color_909090),
                ContextCompat.getColor(requireContext(), R.color.color_3bb9f1)
            )
            .setIndicatorSliderWidth(4f.dpToPx().toInt(), 10f.dpToPx().toInt())
            .setIndicatorHeight(4f.dpToPx().toInt())

        viewModel.contentBannerLiveData.observe(viewLifecycleOwner) {
            if (contentBannerAdapter.itemCount <= 0 && it.isEmpty()) {
                binding.rvBanner.visibility = View.GONE
                binding.indicatorBanner.visibility = View.GONE
            } else {
                binding.rvBanner.visibility = View.VISIBLE
                binding.indicatorBanner.visibility = View.VISIBLE
                binding.rvBanner.refreshData(it)
            }
        }
    }

    private fun setupIntroduceCreator() {
        binding.ivIntroduceCreatorAll.setOnClickListener {
            startActivity(
                Intent(requireContext(), IntroduceCreatorActivity::class.java)
            )
        }

        introduceCreatorAdapter = AudioContentMainContentAdapter(
            onClickItem = {
                startActivity(
                    Intent(requireContext(), AudioContentDetailActivity::class.java).apply {
                        putExtra(Constants.EXTRA_AUDIO_CONTENT_ID, it)
                    }
                )
            },
            onClickCreator = {
                startActivity(
                    Intent(requireContext(), UserProfileActivity::class.java).apply {
                        putExtra(Constants.EXTRA_USER_ID, it)
                    }
                )
            }
        )

        val recyclerView = binding.rvIntroduceCreator
        recyclerView.layoutManager = LinearLayoutManager(
            context,
            LinearLayoutManager.HORIZONTAL,
            false
        )

        recyclerView.addItemDecoration(object : RecyclerView.ItemDecoration() {
            override fun getItemOffsets(
                outRect: Rect,
                view: View,
                parent: RecyclerView,
                state: RecyclerView.State
            ) {
                super.getItemOffsets(outRect, view, parent, state)

                when (parent.getChildAdapterPosition(view)) {
                    0 -> {
                        outRect.left = 0
                        outRect.right = 6.7f.dpToPx().toInt()
                    }

                    introduceCreatorAdapter.itemCount - 1 -> {
                        outRect.left = 6.7f.dpToPx().toInt()
                        outRect.right = 0
                    }

                    else -> {
                        outRect.left = 6.7f.dpToPx().toInt()
                        outRect.right = 6.7f.dpToPx().toInt()
                    }
                }
            }
        })

        recyclerView.adapter = introduceCreatorAdapter

        viewModel.introduceCreatorLiveData.observe(viewLifecycleOwner) {
            if (it.items.isNotEmpty()) {
                binding.llIntroduceCreator.visibility = View.VISIBLE
                binding.tvIntroduceCreator.text = it.title
                introduceCreatorAdapter.addItems(it.items)
            } else {
                binding.llIntroduceCreator.visibility = View.GONE
            }
        }
    }

    private fun setupRecommendSeries() {
        recommendSeriesAdapter = AudioContentMainNewSeriesAdapter(
            onClickItem = {
                startActivity(
                    Intent(requireContext(), SeriesDetailActivity::class.java).apply {
                        putExtra(Constants.EXTRA_SERIES_ID, it)
                    }
                )
            },
            onClickCreator = {
                startActivity(
                    Intent(requireContext(), UserProfileActivity::class.java).apply {
                        putExtra(Constants.EXTRA_USER_ID, it)
                    }
                )
            }
        )

        val recyclerView = binding.rvRecommendSeries
        recyclerView.layoutManager = LinearLayoutManager(
            requireContext(),
            LinearLayoutManager.HORIZONTAL,
            false
        )

        recyclerView.addItemDecoration(object : RecyclerView.ItemDecoration() {
            override fun getItemOffsets(
                outRect: Rect,
                view: View,
                parent: RecyclerView,
                state: RecyclerView.State
            ) {
                super.getItemOffsets(outRect, view, parent, state)

                when (parent.getChildAdapterPosition(view)) {
                    0 -> {
                        outRect.left = 0
                        outRect.right = 6.7f.dpToPx().toInt()
                    }

                    recommendSeriesAdapter.itemCount - 1 -> {
                        outRect.right = 0
                        outRect.left = 6.7f.dpToPx().toInt()
                    }

                    else -> {
                        outRect.left = 6.7f.dpToPx().toInt()
                        outRect.right = 6.7f.dpToPx().toInt()
                    }
                }
            }
        })

        recyclerView.adapter = recommendSeriesAdapter

        viewModel.recommendSeriesListLiveData.observe(viewLifecycleOwner) {
            recommendSeriesAdapter.addItems(it)
            recyclerView.visibility = if (it.isNotEmpty()) {
                View.VISIBLE
            } else {
                View.GONE
            }
        }
    }

    private fun setupNewContentTheme() {
        newContentThemeAdapter = AudioContentMainNewContentThemeAdapter {
            viewModel.getNewFreeContentOfTheme(theme = it)
        }

        binding.rvNewContentTheme.layoutManager = LinearLayoutManager(
            context,
            LinearLayoutManager.HORIZONTAL,
            false
        )

        binding.rvNewContentTheme.addItemDecoration(object : RecyclerView.ItemDecoration() {
            override fun getItemOffsets(
                outRect: Rect,
                view: View,
                parent: RecyclerView,
                state: RecyclerView.State
            ) {
                super.getItemOffsets(outRect, view, parent, state)

                when (parent.getChildAdapterPosition(view)) {
                    0 -> {
                        outRect.left = 0
                        outRect.right = 4f.dpToPx().toInt()
                    }

                    newContentThemeAdapter.itemCount - 1 -> {
                        outRect.left = 4f.dpToPx().toInt()
                        outRect.right = 0
                    }

                    else -> {
                        outRect.left = 4f.dpToPx().toInt()
                        outRect.right = 4f.dpToPx().toInt()
                    }
                }
            }
        })

        binding.rvNewContentTheme.adapter = newContentThemeAdapter

        viewModel.themeListLiveData.observe(viewLifecycleOwner) {
            binding.llNewContent.visibility = View.VISIBLE
            newContentThemeAdapter.addItems(it)
        }
    }

    private fun setupNewContent() {
        binding.ivNewContentAll.setOnClickListener {
            startActivity(
                Intent(
                    requireContext(),
                    AudioContentNewAllActivity::class.java
                ).apply {
                    putExtra(Constants.EXTRA_AUDIO_CONTENT_FREE, true)
                }
            )
        }

        newContentAdapter = AudioContentMainContentAdapter(
            onClickItem = {
                startActivity(
                    Intent(requireContext(), AudioContentDetailActivity::class.java).apply {
                        putExtra(Constants.EXTRA_AUDIO_CONTENT_ID, it)
                    }
                )
            },
            onClickCreator = {
                startActivity(
                    Intent(requireContext(), UserProfileActivity::class.java).apply {
                        putExtra(Constants.EXTRA_USER_ID, it)
                    }
                )
            }
        )

        binding.rvNewContent.layoutManager = LinearLayoutManager(
            context,
            LinearLayoutManager.HORIZONTAL,
            false
        )

        binding.rvNewContent.addItemDecoration(object : RecyclerView.ItemDecoration() {
            override fun getItemOffsets(
                outRect: Rect,
                view: View,
                parent: RecyclerView,
                state: RecyclerView.State
            ) {
                super.getItemOffsets(outRect, view, parent, state)

                when (parent.getChildAdapterPosition(view)) {
                    0 -> {
                        outRect.left = 0
                        outRect.right = 6.7f.dpToPx().toInt()
                    }

                    newContentAdapter.itemCount - 1 -> {
                        outRect.left = 6.7f.dpToPx().toInt()
                        outRect.right = 0
                    }

                    else -> {
                        outRect.left = 6.7f.dpToPx().toInt()
                        outRect.right = 6.7f.dpToPx().toInt()
                    }
                }
            }
        })

        binding.rvNewContent.adapter = newContentAdapter

        viewModel.newContentListLiveData.observe(viewLifecycleOwner) {
            newContentAdapter.addItems(it)
        }
    }

    private fun setupCuration() {
        curationAdapter = AudioContentMainContentCurationAdapter(
            onClickItem = {
                startActivity(
                    Intent(requireContext(), AudioContentDetailActivity::class.java).apply {
                        putExtra(Constants.EXTRA_AUDIO_CONTENT_ID, it)
                    }
                )
            },
            onClickCreator = {
                startActivity(
                    Intent(requireContext(), UserProfileActivity::class.java).apply {
                        putExtra(Constants.EXTRA_USER_ID, it)
                    }
                )
            }
        )

        binding.rvCuration.layoutManager = LinearLayoutManager(
            context,
            LinearLayoutManager.VERTICAL,
            false
        )

        binding.rvCuration.addItemDecoration(object : RecyclerView.ItemDecoration() {
            override fun getItemOffsets(
                outRect: Rect,
                view: View,
                parent: RecyclerView,
                state: RecyclerView.State
            ) {
                super.getItemOffsets(outRect, view, parent, state)

                when (parent.getChildAdapterPosition(view)) {
                    0 -> {
                        outRect.top = 30f.dpToPx().toInt()
                        outRect.bottom = 15f.dpToPx().toInt()
                    }

                    curationAdapter.itemCount - 1 -> {
                        outRect.top = 15f.dpToPx().toInt()
                        outRect.bottom = 30f.dpToPx().toInt()
                    }

                    else -> {
                        outRect.top = 15f.dpToPx().toInt()
                        outRect.bottom = 15f.dpToPx().toInt()
                    }
                }
            }
        })

        binding.rvCuration.adapter = curationAdapter

        viewModel.curationListLiveData.observe(viewLifecycleOwner) {
            curationAdapter.addItems(it)

            binding.rvCuration.visibility = if (curationAdapter.itemCount <= 0 && it.isEmpty()) {
                View.GONE
            } else {
                View.VISIBLE
            }
        }
    }

    private fun bindData() {
        viewModel.toastLiveData.observe(viewLifecycleOwner) {
            it?.let { Toast.makeText(requireContext(), it, Toast.LENGTH_LONG).show() }
        }

        viewModel.isLoading.observe(viewLifecycleOwner) {
            if (it) {
                loadingDialog.show(screenWidth)
            } else {
                loadingDialog.dismiss()
            }
        }
    }
}
