package kr.co.vividnext.sodalive.audio_content.main.v2.free

import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import com.orhanobut.logger.Logger
import io.reactivex.rxjava3.android.schedulers.AndroidSchedulers
import io.reactivex.rxjava3.schedulers.Schedulers
import kr.co.vividnext.sodalive.audio_content.main.GetAudioContentBannerResponse
import kr.co.vividnext.sodalive.audio_content.main.GetAudioContentMainItem
import kr.co.vividnext.sodalive.audio_content.main.v2.GetContentCurationResponse
import kr.co.vividnext.sodalive.audio_content.main.v2.series.GetRecommendSeriesListResponse
import kr.co.vividnext.sodalive.base.BaseViewModel
import kr.co.vividnext.sodalive.common.SharedPreferenceManager

class AudioContentMainTabFreeViewModel(
    private val repository: AudioContentMainTabFreeRepository
) : BaseViewModel() {

    private val _toastLiveData = MutableLiveData<String?>()
    val toastLiveData: LiveData<String?>
        get() = _toastLiveData

    private var _isLoading = MutableLiveData(false)
    val isLoading: LiveData<Boolean>
        get() = _isLoading

    private var _contentBannerLiveData = MutableLiveData<List<GetAudioContentBannerResponse>>()
    val contentBannerLiveData: LiveData<List<GetAudioContentBannerResponse>>
        get() = _contentBannerLiveData

    private var _introduceCreatorLiveData = MutableLiveData<GetContentCurationResponse>()
    val introduceCreatorLiveData: LiveData<GetContentCurationResponse>
        get() = _introduceCreatorLiveData

    private var _recommendSeriesListLiveData =
        MutableLiveData<List<GetRecommendSeriesListResponse>>()
    val recommendSeriesListLiveData: LiveData<List<GetRecommendSeriesListResponse>>
        get() = _recommendSeriesListLiveData

    private var _newContentListLiveData = MutableLiveData<List<GetAudioContentMainItem>>()
    val newContentListLiveData: LiveData<List<GetAudioContentMainItem>>
        get() = _newContentListLiveData

    private var _themeListLiveData = MutableLiveData<List<String>>()
    val themeListLiveData: LiveData<List<String>>
        get() = _themeListLiveData

    private var _curationListLiveData = MutableLiveData<List<GetContentCurationResponse>>()
    val curationListLiveData: LiveData<List<GetContentCurationResponse>>
        get() = _curationListLiveData

    fun fetchData() {
        _isLoading.value = true
        compositeDisposable.add(
            repository.getContentMainFree(token = "Bearer ${SharedPreferenceManager.token}")
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(
                    {
                        if (it.success && it.data != null) {
                            val data = it.data

                            _contentBannerLiveData.value = data.contentBannerList

                            if (data.introduceCreator != null) {
                                _introduceCreatorLiveData.value = data.introduceCreator!!
                            }

                            _recommendSeriesListLiveData.value = data.recommendSeriesList
                            _newContentListLiveData.value = data.newFreeContentList
                            val themeList = listOf("전체").union(data.themeList).toList()
                            _themeListLiveData.value = themeList

                            _curationListLiveData.value = data.curationList
                        } else {
                            if (it.message != null) {
                                _toastLiveData.postValue(it.message)
                            } else {
                                _toastLiveData.postValue(
                                    "알 수 없는 오류가 발생했습니다. 다시 시도해 주세요."
                                )
                            }
                        }

                        _isLoading.value = false
                    },
                    {
                        _isLoading.value = false
                        it.message?.let { message -> Logger.e(message) }
                        _toastLiveData.postValue("알 수 없는 오류가 발생했습니다. 다시 시도해 주세요.")
                    }
                )
        )
    }

    fun getNewFreeContentOfTheme(theme: String) {
        _isLoading.value = true
        compositeDisposable.add(
            repository.getNewContentOfTheme(
                theme = if (theme == "전체") {
                    ""
                } else {
                    theme
                },
                token = "Bearer ${SharedPreferenceManager.token}"
            )
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(
                    {
                        if (it.success && it.data != null) {
                            _newContentListLiveData.value = it.data!!
                        } else {
                            if (it.message != null) {
                                _toastLiveData.postValue(it.message)
                            } else {
                                _toastLiveData.postValue(
                                    "알 수 없는 오류가 발생했습니다. 다시 시도해 주세요."
                                )
                            }
                        }

                        _isLoading.value = false
                    },
                    {
                        it.message?.let { message -> Logger.e(message) }
                        _toastLiveData.postValue("알 수 없는 오류가 발생했습니다. 다시 시도해 주세요.")
                        _isLoading.value = false
                    }
                )
        )
    }
}
