package kr.co.vividnext.sodalive.audio_content.main.v2.home

import android.content.Intent
import android.graphics.Color
import android.graphics.Rect
import android.net.Uri
import android.os.Bundle
import android.text.SpannableString
import android.text.Spanned
import android.text.style.ForegroundColorSpan
import android.view.View
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.TextView
import android.widget.Toast
import androidx.core.content.ContextCompat
import androidx.media3.common.util.UnstableApi
import androidx.recyclerview.widget.GridLayoutManager
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import coil.load
import coil.transform.CircleCropTransformation
import coil.transform.RoundedCornersTransformation
import com.zhpan.bannerview.BaseBannerAdapter
import com.zhpan.indicator.enums.IndicatorSlideMode
import com.zhpan.indicator.enums.IndicatorStyle
import kr.co.vividnext.sodalive.R
import kr.co.vividnext.sodalive.audio_content.all.AudioContentRankingAllActivity
import kr.co.vividnext.sodalive.audio_content.box.AudioContentBoxActivity
import kr.co.vividnext.sodalive.audio_content.detail.AudioContentDetailActivity
import kr.co.vividnext.sodalive.audio_content.main.AudioContentBannerType
import kr.co.vividnext.sodalive.audio_content.main.GetAudioContentRankingItem
import kr.co.vividnext.sodalive.audio_content.main.banner.AudioContentMainBannerAdapter
import kr.co.vividnext.sodalive.audio_content.main.new_content.AudioContentMainNewContentThemeAdapter
import kr.co.vividnext.sodalive.audio_content.main.ranking.AudioContentMainRankingAdapter
import kr.co.vividnext.sodalive.audio_content.main.v2.AudioContentMainActivity
import kr.co.vividnext.sodalive.audio_content.main.v2.AudioContentMainTab
import kr.co.vividnext.sodalive.audio_content.main.v2.ContentRankCreatorAdapter
import kr.co.vividnext.sodalive.audio_content.series.detail.SeriesDetailActivity
import kr.co.vividnext.sodalive.audio_content.upload.AudioContentUploadActivity
import kr.co.vividnext.sodalive.base.BaseFragment
import kr.co.vividnext.sodalive.common.Constants
import kr.co.vividnext.sodalive.common.LoadingDialog
import kr.co.vividnext.sodalive.common.SharedPreferenceManager
import kr.co.vividnext.sodalive.databinding.FragmentAudioContentMainTabHomeBinding
import kr.co.vividnext.sodalive.explorer.ExplorerSectionAdapter
import kr.co.vividnext.sodalive.explorer.profile.UserProfileActivity
import kr.co.vividnext.sodalive.explorer.profile.series.UserProfileSeriesListAdapter
import kr.co.vividnext.sodalive.extensions.dpToPx
import kr.co.vividnext.sodalive.live.event_banner.EventBannerAdapter
import kr.co.vividnext.sodalive.main.MainActivity
import kr.co.vividnext.sodalive.mypage.alarm.AlarmListActivity
import kr.co.vividnext.sodalive.settings.event.EventDetailActivity
import kr.co.vividnext.sodalive.settings.notice.NoticeDetailActivity
import kr.co.vividnext.sodalive.settings.notification.MemberRole
import org.koin.android.ext.android.inject
import kotlin.math.roundToInt

@UnstableApi
class AudioContentMainTabHomeFragment : BaseFragment<FragmentAudioContentMainTabHomeBinding>(
    FragmentAudioContentMainTabHomeBinding::inflate
) {
    private val viewModel: AudioContentMainTabHomeViewModel by inject()

    private lateinit var loadingDialog: LoadingDialog
    private lateinit var rankCreatorAdapter: ExplorerSectionAdapter
    private lateinit var rankSeriesAdapter: UserProfileSeriesListAdapter
    private lateinit var contentBannerAdapter: AudioContentMainBannerAdapter
    private lateinit var rankContentAdapter: AudioContentMainRankingAdapter
    private lateinit var rankContentSortAdapter: AudioContentMainNewContentThemeAdapter
    private lateinit var contentRankCreatorAdapter: ContentRankCreatorAdapter

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        setupView()
        bindData()

        viewModel.fetchData()
    }

    private fun bindData() {
        viewModel.toastLiveData.observe(viewLifecycleOwner) {
            it?.let { Toast.makeText(requireContext(), it, Toast.LENGTH_LONG).show() }
        }

        viewModel.isLoading.observe(viewLifecycleOwner) {
            if (it) {
                loadingDialog.show(screenWidth)
            } else {
                loadingDialog.dismiss()
            }
        }

        viewModel.salesRankContentListLiveData.observe(viewLifecycleOwner) {
            if (it.isNotEmpty()) {
                binding.llNoItems.visibility = View.GONE
                binding.llSalesTop2.visibility = View.VISIBLE

                setSalesRankContent(
                    item = it[0],
                    titleTextView = binding.tvSalesTitle1,
                    coverImageView = binding.ivSales1,
                    creatorTextView = binding.tvSalesCreator1,
                    creatorImageView = binding.ivSalesCreator1
                )

                if (it.size > 1) {
                    binding.llSales2.visibility = View.VISIBLE
                    setSalesRankContent(
                        item = it[1],
                        titleTextView = binding.tvSalesTitle2,
                        coverImageView = binding.ivSales2,
                        creatorTextView = binding.tvSalesCreator2,
                        creatorImageView = binding.ivSalesCreator2
                    )
                }
            }
        }

        viewModel.salesCountRankContentListLiveData.observe(viewLifecycleOwner) {
            if (it.isNotEmpty()) {
                binding.llNoItems.visibility = View.GONE
                binding.llSalesCountTop2.visibility = View.VISIBLE

                setSalesRankContent(
                    item = it[0],
                    titleTextView = binding.tvSalesCountTitle1,
                    coverImageView = binding.ivSalesCount1,
                    creatorTextView = binding.tvSalesCountCreator1,
                    creatorImageView = binding.ivSalesCountCreator1
                )

                if (it.size > 1) {
                    binding.llSalesCount2.visibility = View.VISIBLE
                    setSalesRankContent(
                        item = it[1],
                        titleTextView = binding.tvSalesCountTitle2,
                        coverImageView = binding.ivSalesCount2,
                        creatorTextView = binding.tvSalesCountCreator2,
                        creatorImageView = binding.ivSalesCountCreator2
                    )
                }
            }
        }
    }

    private fun setSalesRankContent(
        item: GetAudioContentRankingItem,
        titleTextView: TextView,
        creatorTextView: TextView,
        coverImageView: ImageView,
        creatorImageView: ImageView
    ) {
        coverImageView.load(item.coverImageUrl) {
            crossfade(true)
            placeholder(R.drawable.bg_placeholder)
            transformations(RoundedCornersTransformation(5.3f.dpToPx()))
        }
        titleTextView.text = item.title
        creatorTextView.text = item.creatorNickname
        creatorImageView.load(item.creatorProfileImageUrl) {
            transformations(CircleCropTransformation())
            placeholder(R.drawable.ic_place_holder)
            crossfade(true)
        }

        coverImageView.setOnClickListener {
            startActivity(
                Intent(requireActivity(), AudioContentDetailActivity::class.java).apply {
                    putExtra(Constants.EXTRA_AUDIO_CONTENT_ID, item.contentId)
                }
            )
        }

        creatorImageView.setOnClickListener {
            startActivity(
                Intent(requireActivity(), UserProfileActivity::class.java).apply {
                    putExtra(Constants.EXTRA_USER_ID, item.creatorId)
                }
            )
        }
    }

    private fun setupView() {
        loadingDialog = LoadingDialog(requireActivity(), layoutInflater)

        if (SharedPreferenceManager.role == MemberRole.CREATOR.name) {
            binding.llUploadContent.visibility = View.VISIBLE
            binding.llUploadContent.setOnClickListener {
                startActivity(
                    Intent(
                        requireActivity(),
                        AudioContentUploadActivity::class.java
                    )
                )
            }
        } else {
            binding.llUploadContent.visibility = View.GONE
        }

        binding.ivContentKeep.setOnClickListener {
            startActivity(
                Intent(
                    requireContext(),
                    AudioContentBoxActivity::class.java
                )
            )
        }

        binding.ivAlarm.setOnClickListener {
            startActivity(
                Intent(
                    requireActivity(),
                    AlarmListActivity::class.java
                )
            )
        }

        binding.flSearchChannel.setOnClickListener {
            (requireActivity() as MainActivity).showSearchBar()
        }

        setupNotice()
        setupContentBanner()
        setupCategory()
        setupRankCreator()
        setupRankSeries()
        setupRankContentSortType()
        setupRankContent()
        setupEventBanner()
        setupPopularContentByCreator()
    }

    private fun setupNotice() {
        viewModel.noticeLiveData.observe(viewLifecycleOwner) { notice ->
            binding.tvNoticeTitle.text = notice.title
            binding.tvDetail.setOnClickListener {
                startActivity(
                    Intent(requireContext(), NoticeDetailActivity::class.java).apply {
                        putExtra(Constants.EXTRA_NOTICE, notice)
                    }
                )
            }
        }
    }

    private fun setupContentBanner() {
        val layoutParams = binding
            .rvBanner
            .layoutParams as LinearLayout.LayoutParams

        val pagerWidth = screenWidth.toDouble() - 26.7f.dpToPx()
        val pagerHeight = (pagerWidth * 0.53).roundToInt()
        layoutParams.width = pagerWidth.roundToInt()
        layoutParams.height = pagerHeight

        contentBannerAdapter = AudioContentMainBannerAdapter(
            requireContext(),
            pagerWidth.roundToInt(),
            pagerHeight
        ) {
            when (it.type) {
                AudioContentBannerType.EVENT -> {
                    startActivity(
                        Intent(requireContext(), EventDetailActivity::class.java).apply {
                            putExtra(Constants.EXTRA_EVENT, it.eventItem!!)
                        }
                    )
                }

                AudioContentBannerType.CREATOR -> {
                    startActivity(
                        Intent(requireContext(), UserProfileActivity::class.java).apply {
                            putExtra(Constants.EXTRA_USER_ID, it.creatorId!!)
                        }
                    )
                }

                AudioContentBannerType.SERIES -> {
                    startActivity(
                        Intent(requireContext(), SeriesDetailActivity::class.java).apply {
                            putExtra(Constants.EXTRA_SERIES_ID, it.seriesId!!)
                        }
                    )
                }

                AudioContentBannerType.LINK -> {
                    startActivity(Intent(Intent.ACTION_VIEW, Uri.parse(it.link!!)))
                }
            }
        }

        binding
            .rvBanner
            .layoutParams = layoutParams

        binding.rvBanner.apply {
            adapter = contentBannerAdapter as BaseBannerAdapter<Any>

            setLifecycleRegistry(lifecycle)
            setScrollDuration(1000)
            setInterval(4 * 1000)
        }.create()

        binding
            .rvBanner
            .setIndicatorView(binding.indicatorBanner)
            .setIndicatorStyle(IndicatorStyle.ROUND_RECT)
            .setIndicatorSlideMode(IndicatorSlideMode.SMOOTH)
            .setIndicatorVisibility(View.GONE)
            .setIndicatorSliderColor(
                ContextCompat.getColor(requireContext(), R.color.color_909090),
                ContextCompat.getColor(requireContext(), R.color.color_3bb9f1)
            )
            .setIndicatorSliderWidth(4f.dpToPx().toInt(), 10f.dpToPx().toInt())
            .setIndicatorHeight(4f.dpToPx().toInt())

        viewModel.contentBannerLiveData.observe(viewLifecycleOwner) {
            if (contentBannerAdapter.itemCount <= 0 && it.isEmpty()) {
                binding.rvBanner.visibility = View.GONE
                binding.indicatorBanner.visibility = View.GONE
            } else {
                binding.rvBanner.visibility = View.VISIBLE
                binding.indicatorBanner.visibility = View.VISIBLE
                binding.rvBanner.refreshData(it)
            }
        }
    }

    private fun setupCategory() {
        binding.rlCategoryAudioBook.setOnClickListener {
            showToast("준비중 입니다.")
        }

        binding.rlCategoryAudioToon.setOnClickListener {
            showToast("준비중 입니다.")
        }

        binding.rlCategorySeries.setOnClickListener {
            startAudioContentMainActivity(AudioContentMainTab.SERIES)
        }

        binding.rlCategoryContent.setOnClickListener {
            startAudioContentMainActivity(AudioContentMainTab.CONTENT)
        }

        binding.rlCategoryAlarm.setOnClickListener {
            startAudioContentMainActivity(AudioContentMainTab.ALARM)
        }

        binding.rlCategoryAsmr.setOnClickListener {
            startAudioContentMainActivity(AudioContentMainTab.ASMR)
        }

        binding.rlCategoryReplay.setOnClickListener {
            startAudioContentMainActivity(AudioContentMainTab.REPLAY)
        }

        binding.rlCategoryFree.setOnClickListener {
            startAudioContentMainActivity(AudioContentMainTab.FREE)
        }
    }

    private fun startAudioContentMainActivity(tab: AudioContentMainTab) {
        startActivity(
            Intent(requireContext(), AudioContentMainActivity::class.java).apply {
                putExtra(
                    Constants.EXTRA_START_TAB_POSITION,
                    tab.ordinal
                )
            }
        )
    }

    private fun setupRankCreator() {
        rankCreatorAdapter = ExplorerSectionAdapter(
            onClickItem = {
                startActivity(
                    Intent(requireContext(), UserProfileActivity::class.java).apply {
                        putExtra(Constants.EXTRA_USER_ID, it)
                    }
                )
            },
            isVisibleRanking = true
        )

        binding.rvCreatorRank.layoutManager = LinearLayoutManager(
            context,
            LinearLayoutManager.HORIZONTAL,
            false
        )

        binding.rvCreatorRank.addItemDecoration(object : RecyclerView.ItemDecoration() {
            override fun getItemOffsets(
                outRect: Rect,
                view: View,
                parent: RecyclerView,
                state: RecyclerView.State
            ) {
                super.getItemOffsets(outRect, view, parent, state)

                when (parent.getChildAdapterPosition(view)) {
                    0 -> {
                        outRect.left = 0
                        outRect.right = 6.7f.dpToPx().toInt()
                    }

                    rankCreatorAdapter.itemCount - 1 -> {
                        outRect.left = 6.7f.dpToPx().toInt()
                        outRect.right = 0
                    }

                    else -> {
                        outRect.left = 6.7f.dpToPx().toInt()
                        outRect.right = 6.7f.dpToPx().toInt()
                    }
                }
            }
        })

        binding.rvCreatorRank.adapter = rankCreatorAdapter

        viewModel.rankCreatorLiveData.observe(viewLifecycleOwner) {
            binding.tvDesc.text = it.desc
            binding.tvCreatorRankTitle.text = if (
                !it.coloredTitle.isNullOrBlank() &&
                !it.color.isNullOrBlank()
            ) {
                val spStr = SpannableString(it.title)

                try {
                    spStr.setSpan(
                        ForegroundColorSpan(
                            Color.parseColor("#${it.color}")
                        ),
                        it.title.indexOf(it.coloredTitle),
                        it.title.indexOf(it.coloredTitle) + it.coloredTitle.length,
                        Spanned.SPAN_EXCLUSIVE_EXCLUSIVE
                    )

                    spStr
                } catch (e: IllegalArgumentException) {
                    it.title
                }
            } else {
                it.title
            }

            rankCreatorAdapter.addItems(it.creators)
            if (rankCreatorAdapter.itemCount <= 0 && it.creators.isEmpty()) {
                binding.llCreatorRank.visibility = View.GONE
                binding.rvCreatorRank.visibility = View.GONE
            } else {
                binding.llCreatorRank.visibility = View.VISIBLE
                binding.rvCreatorRank.visibility = View.VISIBLE
            }
        }
    }

    private fun setupRankSeries() {
        rankSeriesAdapter = UserProfileSeriesListAdapter(
            onClickItem = {
                startActivity(
                    Intent(requireContext(), SeriesDetailActivity::class.java).apply {
                        putExtra(Constants.EXTRA_SERIES_ID, it)
                    }
                )
            },
            onClickCreator = {
                startActivity(
                    Intent(requireContext(), UserProfileActivity::class.java).apply {
                        putExtra(Constants.EXTRA_USER_ID, it)
                    }
                )
            },
            isVisibleCreator = true
        )

        val recyclerView = binding.rvRankSeries
        recyclerView.layoutManager = LinearLayoutManager(
            requireContext(),
            LinearLayoutManager.HORIZONTAL,
            false
        )

        recyclerView.addItemDecoration(object : RecyclerView.ItemDecoration() {
            override fun getItemOffsets(
                outRect: Rect,
                view: View,
                parent: RecyclerView,
                state: RecyclerView.State
            ) {
                super.getItemOffsets(outRect, view, parent, state)

                when (parent.getChildAdapterPosition(view)) {
                    0 -> {
                        outRect.left = 0
                        outRect.right = 6.7f.dpToPx().toInt()
                    }

                    rankSeriesAdapter.itemCount - 1 -> {
                        outRect.right = 0
                        outRect.left = 6.7f.dpToPx().toInt()
                    }

                    else -> {
                        outRect.left = 6.7f.dpToPx().toInt()
                        outRect.right = 6.7f.dpToPx().toInt()
                    }
                }
            }
        })

        recyclerView.adapter = rankSeriesAdapter

        viewModel.rankSeriesLiveData.observe(viewLifecycleOwner) {
            rankSeriesAdapter.addItems(it)
            binding.llRankSeries.visibility = if (
                rankSeriesAdapter.itemCount <= 0 && it.isEmpty()
            ) {
                View.GONE
            } else {
                View.VISIBLE
            }
        }
    }

    private fun setupRankContentSortType() {
        rankContentSortAdapter = AudioContentMainNewContentThemeAdapter {
            viewModel.getContentRanking(sort = it)
        }

        binding.rvRankContentSort.layoutManager = LinearLayoutManager(
            context,
            LinearLayoutManager.HORIZONTAL,
            false
        )

        binding.rvRankContentSort.addItemDecoration(object : RecyclerView.ItemDecoration() {
            override fun getItemOffsets(
                outRect: Rect,
                view: View,
                parent: RecyclerView,
                state: RecyclerView.State
            ) {
                super.getItemOffsets(outRect, view, parent, state)

                when (parent.getChildAdapterPosition(view)) {
                    0 -> {
                        outRect.left = 0
                        outRect.right = 4f.dpToPx().toInt()
                    }

                    rankContentSortAdapter.itemCount - 1 -> {
                        outRect.left = 4f.dpToPx().toInt()
                        outRect.right = 0
                    }

                    else -> {
                        outRect.left = 4f.dpToPx().toInt()
                        outRect.right = 4f.dpToPx().toInt()
                    }
                }
            }
        })

        binding.rvRankContentSort.adapter = rankContentSortAdapter

        viewModel.rankContentSortListLiveData.observe(viewLifecycleOwner) {
            binding.llRankContent.visibility = View.VISIBLE
            rankContentSortAdapter.addItems(it)
        }
    }

    private fun setupRankContent() {
        binding.ivRankContentAll.setOnClickListener {
            startActivity(Intent(requireContext(), AudioContentRankingAllActivity::class.java))
        }

        rankContentAdapter = AudioContentMainRankingAdapter(
            width = (screenWidth * 0.66).toInt()
        ) {
            startActivity(
                Intent(requireContext(), AudioContentDetailActivity::class.java).apply {
                    putExtra(Constants.EXTRA_AUDIO_CONTENT_ID, it)
                }
            )
        }

        binding.rvRankContent.layoutManager = GridLayoutManager(
            context,
            3,
            GridLayoutManager.HORIZONTAL,
            false
        )

        binding.rvRankContent.addItemDecoration(object : RecyclerView.ItemDecoration() {
            override fun getItemOffsets(
                outRect: Rect,
                view: View,
                parent: RecyclerView,
                state: RecyclerView.State
            ) {
                super.getItemOffsets(outRect, view, parent, state)
                outRect.top = 13.3f.dpToPx().toInt()
                outRect.bottom = 13.3f.dpToPx().toInt()
                outRect.left = 13.3f.dpToPx().toInt()
                outRect.right = 13.3f.dpToPx().toInt()
            }
        })

        binding.rvRankContent.adapter = rankContentAdapter

        viewModel.rankContentLiveData.observe(viewLifecycleOwner) {
            binding.llUploadContent.visibility = View.VISIBLE
            rankContentAdapter.addItems(it)
        }
    }

    private fun setupEventBanner() {
        val imageSliderLp = binding.eventBannerSlider.layoutParams
        imageSliderLp.width = screenWidth
        imageSliderLp.height = (screenWidth * 300) / 1000
        binding.eventBannerSlider.layoutParams = imageSliderLp

        binding.eventBannerSlider.apply {
            adapter = EventBannerAdapter(requireContext()) {
                if (it.detailImageUrl != null) {
                    val intent = Intent(requireActivity(), EventDetailActivity::class.java)
                    intent.putExtra(Constants.EXTRA_EVENT, it)
                    startActivity(intent)
                } else if (!it.link.isNullOrBlank()) {
                    startActivity(
                        Intent(
                            Intent.ACTION_VIEW,
                            Uri.parse(it.link)
                        )
                    )
                }
            } as BaseBannerAdapter<Any>
            setLifecycleRegistry(lifecycle)
            setScrollDuration(800)
        }.create()

        binding.eventBannerSlider
            .setIndicatorView(binding.indicatorEventBanner)
            .setIndicatorStyle(IndicatorStyle.ROUND_RECT)
            .setIndicatorSlideMode(IndicatorSlideMode.SMOOTH)
            .setIndicatorVisibility(View.GONE)
            .setIndicatorSliderColor(
                ContextCompat.getColor(requireContext(), R.color.color_909090),
                ContextCompat.getColor(requireContext(), R.color.color_3bb9f1)
            )
            .setIndicatorSliderWidth(4f.dpToPx().toInt(), 10f.dpToPx().toInt())
            .setIndicatorHeight(4f.dpToPx().toInt())

        viewModel.eventLiveData.observe(viewLifecycleOwner) {
            if (it.isNotEmpty()) {
                binding.eventBannerSlider.visibility = View.VISIBLE
                binding.eventBannerSlider.refreshData(it)
            } else {
                binding.eventBannerSlider.visibility = View.GONE
            }
        }
    }

    private fun setupPopularContentByCreator() {
        contentRankCreatorAdapter = ContentRankCreatorAdapter {
            loadingPopularContentByCreator()
            viewModel.getPopularContentByCreator(it)
        }

        binding.rvRankingCreator.layoutManager = LinearLayoutManager(
            context,
            LinearLayoutManager.HORIZONTAL,
            false
        )

        binding.rvRankingCreator.addItemDecoration(object : RecyclerView.ItemDecoration() {
            override fun getItemOffsets(
                outRect: Rect,
                view: View,
                parent: RecyclerView,
                state: RecyclerView.State
            ) {
                super.getItemOffsets(outRect, view, parent, state)

                when (parent.getChildAdapterPosition(view)) {
                    0 -> {
                        outRect.left = 0
                        outRect.right = 4f.dpToPx().toInt()
                    }

                    contentRankCreatorAdapter.itemCount - 1 -> {
                        outRect.left = 4f.dpToPx().toInt()
                        outRect.right = 0
                    }

                    else -> {
                        outRect.left = 4f.dpToPx().toInt()
                        outRect.right = 4f.dpToPx().toInt()
                    }
                }
            }
        })

        binding.rvRankingCreator.adapter = contentRankCreatorAdapter

        viewModel.contentRankCreatorListLiveData.observe(viewLifecycleOwner) {
            contentRankCreatorAdapter.addItems(it)
            if (contentRankCreatorAdapter.itemCount <= 0 && it.isEmpty()) {
                binding.llCreatorContentRanking.visibility = View.GONE
            } else {
                binding.llCreatorContentRanking.visibility = View.VISIBLE
            }
        }
    }

    private fun loadingPopularContentByCreator() {
        binding.llSales2.visibility = View.GONE
        binding.llSalesTop2.visibility = View.GONE
        binding.llSalesCount2.visibility = View.GONE
        binding.llSalesCountTop2.visibility = View.GONE
        binding.llNoItems.visibility = View.VISIBLE
    }
}
