package kr.co.vividnext.sodalive.audio_content.main.v2.series

import android.content.Intent
import android.graphics.Rect
import android.net.Uri
import android.os.Bundle
import android.view.View
import android.widget.LinearLayout
import android.widget.Toast
import androidx.core.content.ContextCompat
import androidx.recyclerview.widget.GridLayoutManager
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.zhpan.bannerview.BaseBannerAdapter
import com.zhpan.indicator.enums.IndicatorSlideMode
import com.zhpan.indicator.enums.IndicatorStyle
import kr.co.vividnext.sodalive.R
import kr.co.vividnext.sodalive.audio_content.main.AudioContentBannerType
import kr.co.vividnext.sodalive.audio_content.main.banner.AudioContentMainBannerAdapter
import kr.co.vividnext.sodalive.audio_content.main.v2.ContentRankCreatorAdapter
import kr.co.vividnext.sodalive.audio_content.main.v2.series.completed.CompletedSeriesActivity
import kr.co.vividnext.sodalive.audio_content.main.v2.series.curation.AudioContentMainSeriesCurationAdapter
import kr.co.vividnext.sodalive.audio_content.main.v2.series.new_series.AudioContentMainNewSeriesAdapter
import kr.co.vividnext.sodalive.audio_content.main.v2.series.origianl_audio_drama.AudioContentMainTabSeriesOriginalAudioDramaAdapter
import kr.co.vividnext.sodalive.audio_content.main.v2.series.origianl_audio_drama.OriginalAudioDramaContentAllActivity
import kr.co.vividnext.sodalive.audio_content.main.v2.series.rank_series.AudioContentMainSeriesRankingAdapter
import kr.co.vividnext.sodalive.audio_content.main.v2.series.recommend_by_genre.AudioContentMainRecommendSeriesGenreAdapter
import kr.co.vividnext.sodalive.audio_content.series.detail.SeriesDetailActivity
import kr.co.vividnext.sodalive.base.BaseFragment
import kr.co.vividnext.sodalive.common.Constants
import kr.co.vividnext.sodalive.common.LoadingDialog
import kr.co.vividnext.sodalive.databinding.FragmentAudioContentMainTabSeriesBinding
import kr.co.vividnext.sodalive.explorer.profile.UserProfileActivity
import kr.co.vividnext.sodalive.explorer.profile.series.UserProfileSeriesListAdapter
import kr.co.vividnext.sodalive.extensions.dpToPx
import kr.co.vividnext.sodalive.live.event_banner.EventBannerAdapter
import kr.co.vividnext.sodalive.settings.event.EventDetailActivity
import org.koin.android.ext.android.inject
import kotlin.math.roundToInt

class AudioContentMainTabSeriesFragment : BaseFragment<FragmentAudioContentMainTabSeriesBinding>(
    FragmentAudioContentMainTabSeriesBinding::inflate
) {
    private val viewModel: AudioContentMainTabSeriesViewModel by inject()

    private lateinit var loadingDialog: LoadingDialog
    private lateinit var contentBannerAdapter: AudioContentMainBannerAdapter
    private lateinit var audioDramaAdapter: AudioContentMainTabSeriesOriginalAudioDramaAdapter
    private lateinit var rankDailySeriesAdapter: AudioContentMainSeriesRankingAdapter
    private lateinit var seriesGenreAdapter: AudioContentMainRecommendSeriesGenreAdapter
    private lateinit var recommendSeriesByGenreAdapter: UserProfileSeriesListAdapter
    private lateinit var newSeriesAdapter: AudioContentMainNewSeriesAdapter
    private lateinit var completedSeriesAdapter: UserProfileSeriesListAdapter
    private lateinit var recommendSeriesCreatorAdapter: ContentRankCreatorAdapter
    private lateinit var recommendSeriesByChannelAdapter: UserProfileSeriesListAdapter
    private lateinit var curationAdapter: AudioContentMainSeriesCurationAdapter

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        setupView()
        bindData()

        viewModel.fetchData()
    }

    private fun setupView() {
        loadingDialog = LoadingDialog(requireActivity(), layoutInflater)

        setupContentBanner()
        setupOriginalAudioDrama()
        setupRankSeries()
        setupRecommendSeriesGenre()
        setupRecommendSeriesByGenre()
        setupNewSeries()
        setupCompleteSeries()
        setupRecommendSeriesByChannelCreator()
        setupRecommendSeriesByChannel()
        setupEventBanner()
        setupCuration()
    }

    private fun setupContentBanner() {
        val layoutParams = binding
            .rvBanner
            .layoutParams as LinearLayout.LayoutParams

        val pagerWidth = screenWidth.toDouble() - 26.7f.dpToPx()
        val pagerHeight = (pagerWidth * 0.53).roundToInt()
        layoutParams.width = pagerWidth.roundToInt()
        layoutParams.height = pagerHeight

        contentBannerAdapter = AudioContentMainBannerAdapter(
            requireContext(),
            pagerWidth.roundToInt(),
            pagerHeight
        ) {
            when (it.type) {
                AudioContentBannerType.EVENT -> {
                    startActivity(
                        Intent(requireContext(), EventDetailActivity::class.java).apply {
                            putExtra(Constants.EXTRA_EVENT, it.eventItem!!)
                        }
                    )
                }

                AudioContentBannerType.CREATOR -> {
                    startActivity(
                        Intent(requireContext(), UserProfileActivity::class.java).apply {
                            putExtra(Constants.EXTRA_USER_ID, it.creatorId!!)
                        }
                    )
                }

                AudioContentBannerType.SERIES -> {
                    startActivity(
                        Intent(requireContext(), SeriesDetailActivity::class.java).apply {
                            putExtra(Constants.EXTRA_SERIES_ID, it.seriesId!!)
                        }
                    )
                }

                AudioContentBannerType.LINK -> {
                    startActivity(Intent(Intent.ACTION_VIEW, Uri.parse(it.link!!)))
                }
            }
        }

        binding
            .rvBanner
            .layoutParams = layoutParams

        binding.rvBanner.apply {
            adapter = contentBannerAdapter as BaseBannerAdapter<Any>

            setLifecycleRegistry(lifecycle)
            setScrollDuration(1000)
            setInterval(4 * 1000)
        }.create()

        binding
            .rvBanner
            .setIndicatorView(binding.indicatorBanner)
            .setIndicatorStyle(IndicatorStyle.ROUND_RECT)
            .setIndicatorSlideMode(IndicatorSlideMode.SMOOTH)
            .setIndicatorVisibility(View.GONE)
            .setIndicatorSliderColor(
                ContextCompat.getColor(requireContext(), R.color.color_909090),
                ContextCompat.getColor(requireContext(), R.color.color_3bb9f1)
            )
            .setIndicatorSliderWidth(4f.dpToPx().toInt(), 10f.dpToPx().toInt())
            .setIndicatorHeight(4f.dpToPx().toInt())

        viewModel.contentBannerLiveData.observe(viewLifecycleOwner) {
            if (contentBannerAdapter.itemCount <= 0 && it.isEmpty()) {
                binding.rvBanner.visibility = View.GONE
                binding.indicatorBanner.visibility = View.GONE
            } else {
                binding.rvBanner.visibility = View.VISIBLE
                binding.indicatorBanner.visibility = View.VISIBLE
                binding.rvBanner.refreshData(it)
            }
        }
    }

    private fun setupOriginalAudioDrama() {
        audioDramaAdapter = AudioContentMainTabSeriesOriginalAudioDramaAdapter {
            startActivity(
                Intent(requireContext(), SeriesDetailActivity::class.java).apply {
                    putExtra(Constants.EXTRA_SERIES_ID, it)
                }
            )
        }

        val recyclerView = binding.rvOriginalAudio
        recyclerView.layoutManager = LinearLayoutManager(
            requireContext(),
            LinearLayoutManager.HORIZONTAL,
            false
        )

        recyclerView.addItemDecoration(object : RecyclerView.ItemDecoration() {
            override fun getItemOffsets(
                outRect: Rect,
                view: View,
                parent: RecyclerView,
                state: RecyclerView.State
            ) {
                super.getItemOffsets(outRect, view, parent, state)

                when (parent.getChildAdapterPosition(view)) {
                    0 -> {
                        outRect.left = 0
                        outRect.right = 6.7f.dpToPx().toInt()
                    }

                    audioDramaAdapter.itemCount - 1 -> {
                        outRect.right = 0
                        outRect.left = 6.7f.dpToPx().toInt()
                    }

                    else -> {
                        outRect.left = 6.7f.dpToPx().toInt()
                        outRect.right = 6.7f.dpToPx().toInt()
                    }
                }
            }
        })

        recyclerView.adapter = audioDramaAdapter

        viewModel.originalAudioDramaLiveData.observe(viewLifecycleOwner) {
            audioDramaAdapter.addItems(it)
            binding.llOriginalAudioDrama.visibility =
                if (audioDramaAdapter.isVisibleRecyclerView()) {
                    View.VISIBLE
                } else {
                    View.GONE
                }
        }

        binding.ivOriginalAudioDramaAll.setOnClickListener {
            startActivity(
                Intent(
                    requireContext(),
                    OriginalAudioDramaContentAllActivity::class.java
                )
            )
        }
    }

    private fun setupRankSeries() {
        rankDailySeriesAdapter = AudioContentMainSeriesRankingAdapter {
            startActivity(
                Intent(requireContext(), SeriesDetailActivity::class.java).apply {
                    putExtra(Constants.EXTRA_SERIES_ID, it)
                }
            )
        }

        binding.rvRankSeries.layoutManager = GridLayoutManager(
            context,
            3,
            GridLayoutManager.HORIZONTAL,
            false
        )

        binding.rvRankSeries.addItemDecoration(object : RecyclerView.ItemDecoration() {
            override fun getItemOffsets(
                outRect: Rect,
                view: View,
                parent: RecyclerView,
                state: RecyclerView.State
            ) {
                super.getItemOffsets(outRect, view, parent, state)
                outRect.top = 13.3f.dpToPx().toInt()
                outRect.bottom = 13.3f.dpToPx().toInt()
                outRect.left = 13.3f.dpToPx().toInt()
                outRect.right = 13.3f.dpToPx().toInt()
            }
        })

        binding.rvRankSeries.adapter = rankDailySeriesAdapter

        viewModel.rankSeriesListLiveData.observe(viewLifecycleOwner) {
            if (it.isNotEmpty()) {
                binding.llRankSeries.visibility = View.VISIBLE
                rankDailySeriesAdapter.addItems(it)
            } else {
                binding.llRankSeries.visibility = View.GONE
            }
        }
    }

    private fun setupRecommendSeriesGenre() {
        seriesGenreAdapter = AudioContentMainRecommendSeriesGenreAdapter {
            binding.llNoItemsSeriesByGenre.visibility = View.VISIBLE
            binding.rvSeriesByGenre.visibility = View.GONE
            viewModel.selectGenre(it)
        }

        val recyclerView = binding.rvSeriesGenre
        recyclerView.layoutManager = LinearLayoutManager(
            requireContext(),
            LinearLayoutManager.HORIZONTAL,
            false
        )

        recyclerView.addItemDecoration(object : RecyclerView.ItemDecoration() {
            override fun getItemOffsets(
                outRect: Rect,
                view: View,
                parent: RecyclerView,
                state: RecyclerView.State
            ) {
                super.getItemOffsets(outRect, view, parent, state)

                when (parent.getChildAdapterPosition(view)) {
                    0 -> {
                        outRect.left = 0
                        outRect.right = 4f.dpToPx().toInt()
                    }

                    seriesGenreAdapter.itemCount - 1 -> {
                        outRect.left = 4f.dpToPx().toInt()
                        outRect.right = 0
                    }

                    else -> {
                        outRect.left = 4f.dpToPx().toInt()
                        outRect.right = 4f.dpToPx().toInt()
                    }
                }
            }
        })

        recyclerView.adapter = seriesGenreAdapter

        viewModel.genreListLiveData.observe(viewLifecycleOwner) {
            if (it.isNotEmpty()) {
                binding.llSeriesByGenre.visibility = View.VISIBLE
                seriesGenreAdapter.addItems(it)
            } else {
                binding.llSeriesByGenre.visibility = View.GONE
            }
        }
    }

    private fun setupRecommendSeriesByGenre() {
        recommendSeriesByGenreAdapter = UserProfileSeriesListAdapter(
            onClickItem = {
                startActivity(
                    Intent(requireContext(), SeriesDetailActivity::class.java).apply {
                        putExtra(Constants.EXTRA_SERIES_ID, it)
                    }
                )
            },
            onClickCreator = {
                startActivity(
                    Intent(requireContext(), UserProfileActivity::class.java).apply {
                        putExtra(Constants.EXTRA_USER_ID, it)
                    }
                )
            },
            isVisibleCreator = true
        )

        val recyclerView = binding.rvSeriesByGenre
        recyclerView.layoutManager = LinearLayoutManager(
            requireContext(),
            LinearLayoutManager.HORIZONTAL,
            false
        )

        recyclerView.addItemDecoration(object : RecyclerView.ItemDecoration() {
            override fun getItemOffsets(
                outRect: Rect,
                view: View,
                parent: RecyclerView,
                state: RecyclerView.State
            ) {
                super.getItemOffsets(outRect, view, parent, state)

                when (parent.getChildAdapterPosition(view)) {
                    0 -> {
                        outRect.left = 0
                        outRect.right = 6.7f.dpToPx().toInt()
                    }

                    recommendSeriesByGenreAdapter.itemCount - 1 -> {
                        outRect.right = 0
                        outRect.left = 6.7f.dpToPx().toInt()
                    }

                    else -> {
                        outRect.left = 6.7f.dpToPx().toInt()
                        outRect.right = 6.7f.dpToPx().toInt()
                    }
                }
            }
        })

        recyclerView.adapter = recommendSeriesByGenreAdapter

        viewModel.recommendSeriesListLiveData.observe(viewLifecycleOwner) {
            if (it.isNotEmpty()) {
                binding.llNoItemsSeriesByGenre.visibility = View.GONE
                binding.rvSeriesByGenre.visibility = View.VISIBLE
                recommendSeriesByGenreAdapter.clear()
                recommendSeriesByGenreAdapter.addItems(it)
            }
        }
    }

    private fun setupNewSeries() {
        newSeriesAdapter = AudioContentMainNewSeriesAdapter(
            onClickItem = {
                startActivity(
                    Intent(requireContext(), SeriesDetailActivity::class.java).apply {
                        putExtra(Constants.EXTRA_SERIES_ID, it)
                    }
                )
            },
            onClickCreator = {
                startActivity(
                    Intent(requireContext(), UserProfileActivity::class.java).apply {
                        putExtra(Constants.EXTRA_USER_ID, it)
                    }
                )
            }
        )

        val recyclerView = binding.rvNewSeries
        recyclerView.layoutManager = LinearLayoutManager(
            requireContext(),
            LinearLayoutManager.HORIZONTAL,
            false
        )

        recyclerView.addItemDecoration(object : RecyclerView.ItemDecoration() {
            override fun getItemOffsets(
                outRect: Rect,
                view: View,
                parent: RecyclerView,
                state: RecyclerView.State
            ) {
                super.getItemOffsets(outRect, view, parent, state)

                when (parent.getChildAdapterPosition(view)) {
                    0 -> {
                        outRect.left = 0
                        outRect.right = 6.7f.dpToPx().toInt()
                    }

                    recommendSeriesByGenreAdapter.itemCount - 1 -> {
                        outRect.right = 0
                        outRect.left = 6.7f.dpToPx().toInt()
                    }

                    else -> {
                        outRect.left = 6.7f.dpToPx().toInt()
                        outRect.right = 6.7f.dpToPx().toInt()
                    }
                }
            }
        })

        recyclerView.adapter = newSeriesAdapter

        viewModel.newSeriesListLiveData.observe(viewLifecycleOwner) {
            newSeriesAdapter.addItems(it)
            binding.llNewSeries.visibility = if (it.isNotEmpty()) {
                View.VISIBLE
            } else {
                View.GONE
            }
        }
    }

    private fun setupCompleteSeries() {
        binding.ivCompleteSeriesAll.setOnClickListener {
            startActivity(
                Intent(
                    requireContext(),
                    CompletedSeriesActivity::class.java
                )
            )
        }

        completedSeriesAdapter = UserProfileSeriesListAdapter(
            onClickItem = {
                startActivity(
                    Intent(requireContext(), SeriesDetailActivity::class.java).apply {
                        putExtra(Constants.EXTRA_SERIES_ID, it)
                    }
                )
            },
            onClickCreator = {
                startActivity(
                    Intent(requireContext(), UserProfileActivity::class.java).apply {
                        putExtra(Constants.EXTRA_USER_ID, it)
                    }
                )
            },
            isVisibleCreator = true
        )

        val recyclerView = binding.rvRankSeries
        recyclerView.layoutManager = LinearLayoutManager(
            requireContext(),
            LinearLayoutManager.HORIZONTAL,
            false
        )

        recyclerView.addItemDecoration(object : RecyclerView.ItemDecoration() {
            override fun getItemOffsets(
                outRect: Rect,
                view: View,
                parent: RecyclerView,
                state: RecyclerView.State
            ) {
                super.getItemOffsets(outRect, view, parent, state)

                when (parent.getChildAdapterPosition(view)) {
                    0 -> {
                        outRect.left = 0
                        outRect.right = 6.7f.dpToPx().toInt()
                    }

                    completedSeriesAdapter.itemCount - 1 -> {
                        outRect.right = 0
                        outRect.left = 6.7f.dpToPx().toInt()
                    }

                    else -> {
                        outRect.left = 6.7f.dpToPx().toInt()
                        outRect.right = 6.7f.dpToPx().toInt()
                    }
                }
            }
        })

        recyclerView.adapter = completedSeriesAdapter

        viewModel.rankCompleteSeriesListLiveData.observe(viewLifecycleOwner) {
            completedSeriesAdapter.addItems(it)
            binding.llCompleteSeries.visibility = if (
                completedSeriesAdapter.itemCount <= 0 && it.isEmpty()
            ) {
                View.GONE
            } else {
                View.VISIBLE
            }
        }
    }

    private fun setupRecommendSeriesByChannelCreator() {
        recommendSeriesCreatorAdapter = ContentRankCreatorAdapter {
            binding.llNoItems.visibility = View.VISIBLE
            binding.rvRecommendSeriesByChannel.visibility = View.GONE
            viewModel.getRecommendSeriesByCreator(it)
        }

        val recyclerView = binding.rvRecommendSeriesChannel
        recyclerView.layoutManager = LinearLayoutManager(
            context,
            LinearLayoutManager.HORIZONTAL,
            false
        )

        recyclerView.addItemDecoration(object : RecyclerView.ItemDecoration() {
            override fun getItemOffsets(
                outRect: Rect,
                view: View,
                parent: RecyclerView,
                state: RecyclerView.State
            ) {
                super.getItemOffsets(outRect, view, parent, state)

                when (parent.getChildAdapterPosition(view)) {
                    0 -> {
                        outRect.left = 0
                        outRect.right = 4f.dpToPx().toInt()
                    }

                    recommendSeriesCreatorAdapter.itemCount - 1 -> {
                        outRect.left = 4f.dpToPx().toInt()
                        outRect.right = 0
                    }

                    else -> {
                        outRect.left = 4f.dpToPx().toInt()
                        outRect.right = 4f.dpToPx().toInt()
                    }
                }
            }
        })

        recyclerView.adapter = recommendSeriesCreatorAdapter

        viewModel.seriesRankCreatorListLiveData.observe(viewLifecycleOwner) {
            recommendSeriesCreatorAdapter.addItems(it)
            if (recommendSeriesCreatorAdapter.itemCount <= 0 && it.isEmpty()) {
                binding.llRecommendSeriesByChannel.visibility = View.GONE
            } else {
                binding.llRecommendSeriesByChannel.visibility = View.VISIBLE
            }
        }
    }

    private fun setupRecommendSeriesByChannel() {
        recommendSeriesByChannelAdapter = UserProfileSeriesListAdapter(
            onClickItem = {
                startActivity(
                    Intent(requireContext(), SeriesDetailActivity::class.java).apply {
                        putExtra(Constants.EXTRA_SERIES_ID, it)
                    }
                )
            },
            onClickCreator = {
                startActivity(
                    Intent(requireContext(), UserProfileActivity::class.java).apply {
                        putExtra(Constants.EXTRA_USER_ID, it)
                    }
                )
            },
            isVisibleCreator = true
        )

        val recyclerView = binding.rvRecommendSeriesByChannel
        recyclerView.layoutManager = LinearLayoutManager(
            requireContext(),
            LinearLayoutManager.HORIZONTAL,
            false
        )

        recyclerView.addItemDecoration(object : RecyclerView.ItemDecoration() {
            override fun getItemOffsets(
                outRect: Rect,
                view: View,
                parent: RecyclerView,
                state: RecyclerView.State
            ) {
                super.getItemOffsets(outRect, view, parent, state)

                when (parent.getChildAdapterPosition(view)) {
                    0 -> {
                        outRect.left = 0
                        outRect.right = 6.7f.dpToPx().toInt()
                    }

                    recommendSeriesByChannelAdapter.itemCount - 1 -> {
                        outRect.right = 0
                        outRect.left = 6.7f.dpToPx().toInt()
                    }

                    else -> {
                        outRect.left = 6.7f.dpToPx().toInt()
                        outRect.right = 6.7f.dpToPx().toInt()
                    }
                }
            }
        })

        recyclerView.adapter = recommendSeriesByChannelAdapter

        viewModel.recommendSeriesByChannelLiveData.observe(viewLifecycleOwner) {
            recommendSeriesByChannelAdapter.clear()
            recommendSeriesByChannelAdapter.addItems(it)
            if (recommendSeriesCreatorAdapter.itemCount <= 0) {
                binding.llNoItems.visibility = View.VISIBLE
                binding.rvRecommendSeriesByChannel.visibility = View.GONE
            } else {
                binding.llNoItems.visibility = View.GONE
                binding.rvRecommendSeriesByChannel.visibility = View.VISIBLE
            }
        }
    }

    private fun setupEventBanner() {
        val imageSliderLp = binding.eventBannerSlider.layoutParams
        imageSliderLp.width = screenWidth
        imageSliderLp.height = (screenWidth * 300) / 1000
        binding.eventBannerSlider.layoutParams = imageSliderLp

        binding.eventBannerSlider.apply {
            adapter = EventBannerAdapter(requireContext()) {
                if (it.detailImageUrl != null) {
                    val intent = Intent(requireActivity(), EventDetailActivity::class.java)
                    intent.putExtra(Constants.EXTRA_EVENT, it)
                    startActivity(intent)
                } else if (!it.link.isNullOrBlank()) {
                    startActivity(
                        Intent(
                            Intent.ACTION_VIEW,
                            Uri.parse(it.link)
                        )
                    )
                }
            } as BaseBannerAdapter<Any>
            setLifecycleRegistry(lifecycle)
            setScrollDuration(800)
        }.create()

        binding.eventBannerSlider
            .setIndicatorView(binding.indicatorEventBanner)
            .setIndicatorStyle(IndicatorStyle.ROUND_RECT)
            .setIndicatorSlideMode(IndicatorSlideMode.SMOOTH)
            .setIndicatorVisibility(View.GONE)
            .setIndicatorSliderColor(
                ContextCompat.getColor(requireContext(), R.color.color_909090),
                ContextCompat.getColor(requireContext(), R.color.color_3bb9f1)
            )
            .setIndicatorSliderWidth(4f.dpToPx().toInt(), 10f.dpToPx().toInt())
            .setIndicatorHeight(4f.dpToPx().toInt())

        viewModel.eventLiveData.observe(viewLifecycleOwner) {
            if (it.isNotEmpty()) {
                binding.eventBannerSlider.visibility = View.VISIBLE
                binding.indicatorEventBanner.visibility = View.VISIBLE
                binding.eventBannerSlider.refreshData(it)
            } else {
                binding.eventBannerSlider.visibility = View.GONE
                binding.indicatorEventBanner.visibility = View.GONE
            }
        }
    }

    private fun setupCuration() {
        curationAdapter = AudioContentMainSeriesCurationAdapter(
            onClickItem = {
                startActivity(
                    Intent(requireContext(), SeriesDetailActivity::class.java).apply {
                        putExtra(Constants.EXTRA_SERIES_ID, it)
                    }
                )
            },
            onClickCreator = {
                startActivity(
                    Intent(requireContext(), UserProfileActivity::class.java).apply {
                        putExtra(Constants.EXTRA_USER_ID, it)
                    }
                )
            },
        )

        binding.rvCuration.layoutManager = LinearLayoutManager(
            context,
            LinearLayoutManager.VERTICAL,
            false
        )

        binding.rvCuration.addItemDecoration(object : RecyclerView.ItemDecoration() {
            override fun getItemOffsets(
                outRect: Rect,
                view: View,
                parent: RecyclerView,
                state: RecyclerView.State
            ) {
                super.getItemOffsets(outRect, view, parent, state)

                when (parent.getChildAdapterPosition(view)) {
                    0 -> {
                        outRect.top = 30f.dpToPx().toInt()
                        outRect.bottom = 15f.dpToPx().toInt()
                    }

                    curationAdapter.itemCount - 1 -> {
                        outRect.top = 15f.dpToPx().toInt()
                        outRect.bottom = 30f.dpToPx().toInt()
                    }

                    else -> {
                        outRect.top = 15f.dpToPx().toInt()
                        outRect.bottom = 15f.dpToPx().toInt()
                    }
                }
            }
        })

        binding.rvCuration.adapter = curationAdapter

        viewModel.curationListLiveData.observe(viewLifecycleOwner) {
            curationAdapter.addItems(it)

            binding.rvCuration.visibility = if (curationAdapter.itemCount <= 0 && it.isEmpty()) {
                View.GONE
            } else {
                View.VISIBLE
            }
        }
    }

    private fun bindData() {
        viewModel.toastLiveData.observe(viewLifecycleOwner) {
            it?.let { Toast.makeText(requireContext(), it, Toast.LENGTH_LONG).show() }
        }

        viewModel.isLoading.observe(viewLifecycleOwner) {
            if (it) {
                loadingDialog.show(screenWidth)
            } else {
                loadingDialog.dismiss()
            }
        }
    }
}
