package kr.co.vividnext.sodalive.audio_content.main.v2.series

import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import com.orhanobut.logger.Logger
import io.reactivex.rxjava3.android.schedulers.AndroidSchedulers
import io.reactivex.rxjava3.schedulers.Schedulers
import kr.co.vividnext.sodalive.audio_content.main.ContentCreatorResponse
import kr.co.vividnext.sodalive.audio_content.main.GetAudioContentBannerResponse
import kr.co.vividnext.sodalive.audio_content.series.GetSeriesListResponse
import kr.co.vividnext.sodalive.base.BaseViewModel
import kr.co.vividnext.sodalive.common.SharedPreferenceManager
import kr.co.vividnext.sodalive.settings.event.EventItem

class AudioContentMainTabSeriesViewModel(
    private val repository: AudioContentMainTabSeriesRepository
) : BaseViewModel() {
    private val _toastLiveData = MutableLiveData<String?>()
    val toastLiveData: LiveData<String?>
        get() = _toastLiveData

    private var _isLoading = MutableLiveData(false)
    val isLoading: LiveData<Boolean>
        get() = _isLoading

    private var _contentBannerLiveData = MutableLiveData<List<GetAudioContentBannerResponse>>()
    val contentBannerLiveData: LiveData<List<GetAudioContentBannerResponse>>
        get() = _contentBannerLiveData

    private var _originalAudioDramaLiveData =
        MutableLiveData<List<GetSeriesListResponse.SeriesListItem>>()
    val originalAudioDramaLiveData: LiveData<List<GetSeriesListResponse.SeriesListItem>>
        get() = _originalAudioDramaLiveData

    private var _rankSeriesListLiveData =
        MutableLiveData<List<GetSeriesListResponse.SeriesListItem>>()
    val rankSeriesListLiveData: LiveData<List<GetSeriesListResponse.SeriesListItem>>
        get() = _rankSeriesListLiveData

    private var _genreListLiveData = MutableLiveData<List<GetSeriesGenreListResponse>>()
    val genreListLiveData: LiveData<List<GetSeriesGenreListResponse>>
        get() = _genreListLiveData

    private var _recommendSeriesListLiveData =
        MutableLiveData<List<GetSeriesListResponse.SeriesListItem>>()
    val recommendSeriesListLiveData: LiveData<List<GetSeriesListResponse.SeriesListItem>>
        get() = _recommendSeriesListLiveData

    private var _newSeriesListLiveData = MutableLiveData<List<GetRecommendSeriesListResponse>>()
    val newSeriesListLiveData: LiveData<List<GetRecommendSeriesListResponse>>
        get() = _newSeriesListLiveData

    private var _rankCompleteSeriesListLiveData =
        MutableLiveData<List<GetSeriesListResponse.SeriesListItem>>()
    val rankCompleteSeriesListLiveData: LiveData<List<GetSeriesListResponse.SeriesListItem>>
        get() = _rankCompleteSeriesListLiveData

    private var _seriesRankCreatorListLiveData = MutableLiveData<List<ContentCreatorResponse>>()
    val seriesRankCreatorListLiveData: LiveData<List<ContentCreatorResponse>>
        get() = _seriesRankCreatorListLiveData

    private var _recommendSeriesByChannelLiveData =
        MutableLiveData<List<GetSeriesListResponse.SeriesListItem>>()
    val recommendSeriesByChannelLiveData: LiveData<List<GetSeriesListResponse.SeriesListItem>>
        get() = _recommendSeriesByChannelLiveData

    private val _eventLiveData = MutableLiveData<List<EventItem>>()
    val eventLiveData: LiveData<List<EventItem>>
        get() = _eventLiveData

    private val _curationListLiveData = MutableLiveData<List<GetSeriesCurationResponse>>()
    val curationListLiveData: LiveData<List<GetSeriesCurationResponse>>
        get() = _curationListLiveData

    fun fetchData() {
        _isLoading.value = true
        compositeDisposable.add(
            repository.getContentMainSeries(token = "Bearer ${SharedPreferenceManager.token}")
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(
                    {
                        if (it.success && it.data != null) {
                            val data = it.data

                            _contentBannerLiveData.value = data.contentBannerList
                            _originalAudioDramaLiveData.value = data.originalAudioDrama
                            _rankSeriesListLiveData.value = data.rankSeriesList
                            _genreListLiveData.value = data.genreList
                            _recommendSeriesListLiveData.value = data.recommendSeriesList
                            _newSeriesListLiveData.value = data.newSeriesList
                            _rankCompleteSeriesListLiveData.value = data.rankCompleteSeriesList
                            _seriesRankCreatorListLiveData.value = data.seriesRankCreatorList
                            _recommendSeriesByChannelLiveData.value = data.recommendSeriesByChannel
                            _eventLiveData.value = data.eventBannerList.eventList
                            _curationListLiveData.value = data.curationList

                        } else {
                            if (it.message != null) {
                                _toastLiveData.postValue(it.message)
                            } else {
                                _toastLiveData.postValue(
                                    "알 수 없는 오류가 발생했습니다. 다시 시도해 주세요."
                                )
                            }
                        }

                        _isLoading.value = false
                    },
                    {
                        _isLoading.value = false
                        it.message?.let { message -> Logger.e(message) }
                        _toastLiveData.postValue("알 수 없는 오류가 발생했습니다. 다시 시도해 주세요.")
                    }
                )
        )
    }

    fun selectGenre(genreId: Long) {
        _isLoading.value = true
        compositeDisposable.add(
            repository.getRecommendSeriesListByGenre(
                genreId,
                token = "Bearer ${SharedPreferenceManager.token}"
            )
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(
                    {
                        if (it.success && it.data != null) {
                            _recommendSeriesListLiveData.value = it.data!!
                        } else {
                            if (it.message != null) {
                                _toastLiveData.postValue(it.message)
                            } else {
                                _toastLiveData.postValue(
                                    "알 수 없는 오류가 발생했습니다. 다시 시도해 주세요."
                                )
                            }
                        }

                        _isLoading.value = false
                    },
                    {
                        _isLoading.value = false
                        it.message?.let { message -> Logger.e(message) }
                        _toastLiveData.postValue("알 수 없는 오류가 발생했습니다. 다시 시도해 주세요.")
                    }
                )
        )
    }

    fun getRecommendSeriesByCreator(creatorId: Long) {
        _isLoading.value = true
        compositeDisposable.add(
            repository.getRecommendSeriesByCreator(creatorId, token = "Bearer ${SharedPreferenceManager.token}")
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(
                    {
                        if (it.success && it.data != null) {
                            _recommendSeriesByChannelLiveData.value = it.data!!
                        } else {
                            if (it.message != null) {
                                _toastLiveData.postValue(it.message)
                            } else {
                                _toastLiveData.postValue(
                                    "알 수 없는 오류가 발생했습니다. 다시 시도해 주세요."
                                )
                            }
                        }

                        _isLoading.value = false
                    },
                    {
                        _isLoading.value = false
                        it.message?.let { message -> Logger.e(message) }
                        _toastLiveData.postValue("알 수 없는 오류가 발생했습니다. 다시 시도해 주세요.")
                    }
                )
        )
    }
}
