package kr.co.vividnext.sodalive.audio_content.main.v2.series.completed

import android.content.Intent
import android.os.Bundle
import androidx.recyclerview.widget.GridLayoutManager
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import kr.co.vividnext.sodalive.audio_content.series.SeriesListAdapter
import kr.co.vividnext.sodalive.audio_content.series.detail.SeriesDetailActivity
import kr.co.vividnext.sodalive.base.BaseActivity
import kr.co.vividnext.sodalive.common.Constants
import kr.co.vividnext.sodalive.common.DifferentSpacingItemDecoration
import kr.co.vividnext.sodalive.common.LoadingDialog
import kr.co.vividnext.sodalive.databinding.ActivityCompletedSeriesBinding
import kr.co.vividnext.sodalive.extensions.dpToPx
import org.koin.android.ext.android.inject
import kotlin.math.roundToInt

class CompletedSeriesActivity : BaseActivity<ActivityCompletedSeriesBinding>(
    ActivityCompletedSeriesBinding::inflate
) {

    private val viewModel: CompletedSeriesViewModel by inject()

    private lateinit var loadingDialog: LoadingDialog
    private lateinit var adapter: SeriesListAdapter

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        bindData()

        viewModel.getCompletedSeries()
    }

    override fun setupView() {
        loadingDialog = LoadingDialog(this, layoutInflater)
        binding.toolbar.tvBack.text = "완결 시리즈"
        binding.toolbar.tvBack.setOnClickListener { finish() }

        setupCompletedSeriesListView()
    }

    private fun setupCompletedSeriesListView() {
        val spacing = 13.3f.dpToPx().roundToInt()

        adapter = SeriesListAdapter(
            itemWidth = ((screenWidth - spacing * 3) / 2f).roundToInt(),
            onClickItem = {
                startActivity(
                    Intent(applicationContext, SeriesDetailActivity::class.java).apply {
                        putExtra(Constants.EXTRA_SERIES_ID, it)
                    }
                )
            },
            onClickCreator = {},
            isVisibleCreator = false
        )

        val spanCount = 2
        val recyclerView = binding.rvSeries
        recyclerView.layoutManager = GridLayoutManager(this, spanCount)

        recyclerView.addItemDecoration(
            DifferentSpacingItemDecoration(
                spanCount = spanCount,
                horizontalSpacing = spacing,
                verticalSpacing = spacing,
                includeEdge = true
            )
        )

        recyclerView.addOnScrollListener(object : RecyclerView.OnScrollListener() {
            override fun onScrolled(recyclerView: RecyclerView, dx: Int, dy: Int) {
                super.onScrolled(recyclerView, dx, dy)

                val lastVisibleItemPosition = (recyclerView.layoutManager as LinearLayoutManager?)!!
                    .findLastCompletelyVisibleItemPosition()
                val itemTotalCount = recyclerView.adapter!!.itemCount - 1

                // 스크롤이 끝에 도달했는지 확인
                if (!recyclerView.canScrollVertically(1) &&
                    lastVisibleItemPosition == itemTotalCount
                ) {
                    viewModel.getCompletedSeries()
                }
            }
        })

        recyclerView.adapter = adapter
    }

    private fun bindData() {
        viewModel.toastLiveData.observe(this) {
            it?.let { showToast(it) }
        }

        viewModel.isLoading.observe(this) {
            if (it) {
                loadingDialog.show(screenWidth)
            } else {
                loadingDialog.dismiss()
            }
        }

        viewModel.completedSeriesLiveData.observe(this) {
            adapter.addItems(it)
        }
    }
}
