package kr.co.vividnext.sodalive.audio_content.main.v2.series.curation

import android.annotation.SuppressLint
import android.content.Context
import android.graphics.Rect
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import kr.co.vividnext.sodalive.audio_content.main.v2.series.GetSeriesCurationResponse
import kr.co.vividnext.sodalive.audio_content.series.GetSeriesListResponse
import kr.co.vividnext.sodalive.databinding.ItemAudioContentMainCurationBinding
import kr.co.vividnext.sodalive.explorer.profile.series.UserProfileSeriesListAdapter
import kr.co.vividnext.sodalive.extensions.dpToPx

class AudioContentMainSeriesCurationAdapter(
    private val onClickItem: (Long) -> Unit,
    private val onClickCreator: (Long) -> Unit
) : RecyclerView.Adapter<AudioContentMainSeriesCurationAdapter.ViewHolder>() {

    private val items = mutableListOf<GetSeriesCurationResponse>()

    inner class ViewHolder(
        private val context: Context,
        private val binding: ItemAudioContentMainCurationBinding
    ) : RecyclerView.ViewHolder(binding.root) {
        fun bind(item: GetSeriesCurationResponse) {
            binding.tvDesc.visibility = View.GONE
            binding.ivAll.visibility = View.GONE

            binding.tvTitle.text = item.title
            setSeriesList(item.items)
        }

        private fun setSeriesList(items: List<GetSeriesListResponse.SeriesListItem>) {
            val adapter = UserProfileSeriesListAdapter(
                onClickItem = onClickItem,
                onClickCreator = onClickCreator,
                isVisibleCreator = true
            )

            binding.rvCuration.layoutManager = LinearLayoutManager(
                context,
                LinearLayoutManager.HORIZONTAL,
                false
            )

            if (binding.rvCuration.itemDecorationCount == 0) {
                binding.rvCuration.addItemDecoration(object : RecyclerView.ItemDecoration() {
                    override fun getItemOffsets(
                        outRect: Rect,
                        view: View,
                        parent: RecyclerView,
                        state: RecyclerView.State
                    ) {
                        super.getItemOffsets(outRect, view, parent, state)

                        when (parent.getChildAdapterPosition(view)) {
                            0 -> {
                                outRect.left = 0
                                outRect.right = 6.7f.dpToPx().toInt()
                            }

                            adapter.itemCount - 1 -> {
                                outRect.left = 6.7f.dpToPx().toInt()
                                outRect.right = 0
                            }

                            else -> {
                                outRect.left = 6.7f.dpToPx().toInt()
                                outRect.right = 6.7f.dpToPx().toInt()
                            }
                        }
                    }
                })
            }

            binding.rvCuration.adapter = adapter
            adapter.addItems(items)
        }
    }

    @SuppressLint("NotifyDataSetChanged")
    fun addItems(items: List<GetSeriesCurationResponse>) {
        this.items.clear()
        this.items.addAll(items)
        notifyDataSetChanged()
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int) = ViewHolder(
        parent.context,
        ItemAudioContentMainCurationBinding.inflate(
            LayoutInflater.from(parent.context),
            parent,
            false
        )
    )

    override fun onBindViewHolder(holder: ViewHolder, position: Int) {
        holder.bind(items[position])
    }

    override fun getItemCount() = items.size
}
