package kr.co.vividnext.sodalive.audio_content.main.v2.series.new_series

import android.annotation.SuppressLint
import android.content.Context
import android.view.LayoutInflater
import android.view.ViewGroup
import androidx.recyclerview.widget.RecyclerView
import coil.load
import coil.transform.CircleCropTransformation
import com.bumptech.glide.Glide
import com.bumptech.glide.load.resource.bitmap.CenterCrop
import com.bumptech.glide.load.resource.bitmap.RoundedCorners
import com.bumptech.glide.request.RequestOptions
import kr.co.vividnext.sodalive.R
import kr.co.vividnext.sodalive.audio_content.main.v2.series.GetRecommendSeriesListResponse
import kr.co.vividnext.sodalive.databinding.ItemAudioContentMainNewSeriesBinding
import kr.co.vividnext.sodalive.extensions.dpToPx

class AudioContentMainNewSeriesAdapter(
    private val onClickItem: (Long) -> Unit,
    private val onClickCreator: (Long) -> Unit
) : RecyclerView.Adapter<AudioContentMainNewSeriesAdapter.ViewHolder>() {

    private val items = mutableListOf<GetRecommendSeriesListResponse>()

    inner class ViewHolder(
        private val context: Context,
        private val binding: ItemAudioContentMainNewSeriesBinding
    ) : RecyclerView.ViewHolder(binding.root) {
        fun bind(item: GetRecommendSeriesListResponse) {
            Glide
                .with(context)
                .load(item.imageUrl)
                .apply(
                    RequestOptions().transform(
                        CenterCrop(),
                        RoundedCorners(5f.dpToPx().toInt())
                    )
                )
                .into(binding.ivCover)

            binding.ivCreatorProfile.load(item.creatorProfileImageUrl) {
                crossfade(true)
                placeholder(R.drawable.ic_place_holder)
                transformations(CircleCropTransformation())
            }

            binding.tvTitle.text = item.title
            binding.tvCreatorNickname.text = item.creatorNickname

            binding.root.setOnClickListener { onClickItem(item.seriesId) }
            binding.tvTitle.setOnClickListener { onClickCreator(item.creatorId) }
            binding.tvCreatorNickname.setOnClickListener { onClickCreator(item.creatorId) }
        }
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int) = ViewHolder(
        parent.context,
        ItemAudioContentMainNewSeriesBinding.inflate(
            LayoutInflater.from(parent.context),
            parent,
            false
        )
    )

    override fun getItemCount() = items.size

    override fun onBindViewHolder(holder: ViewHolder, position: Int) {
        holder.bind(items[position])
    }

    @SuppressLint("NotifyDataSetChanged")
    fun addItems(items: List<GetRecommendSeriesListResponse>) {
        this.items.addAll(items)
        notifyDataSetChanged()
    }
}
