package kr.co.vividnext.sodalive.audio_content.main.v2.series.origianl_audio_drama

import android.annotation.SuppressLint
import android.content.Context
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.recyclerview.widget.RecyclerView
import com.bumptech.glide.Glide
import com.bumptech.glide.load.resource.bitmap.CenterCrop
import com.bumptech.glide.load.resource.bitmap.RoundedCorners
import com.bumptech.glide.request.RequestOptions
import kr.co.vividnext.sodalive.audio_content.series.GetSeriesListResponse
import kr.co.vividnext.sodalive.databinding.ItemSeriesOriginalAudioDramaBinding
import kr.co.vividnext.sodalive.extensions.dpToPx

class AudioContentMainTabSeriesOriginalAudioDramaAdapter(
    private val onClickItem: (Long) -> Unit
) : RecyclerView.Adapter<AudioContentMainTabSeriesOriginalAudioDramaAdapter.ViewHolder>() {

    val items = mutableListOf<GetSeriesListResponse.SeriesListItem>()

    inner class ViewHolder(
        private val context: Context,
        private val binding: ItemSeriesOriginalAudioDramaBinding
    ) : RecyclerView.ViewHolder(binding.root) {
        @SuppressLint("SetTextI18n")
        fun bind(item: GetSeriesListResponse.SeriesListItem) {
            Glide
                .with(context)
                .load(item.coverImage)
                .apply(
                    RequestOptions().transform(
                        CenterCrop(),
                        RoundedCorners(5f.dpToPx().toInt())
                    )
                )
                .into(binding.ivCover)

            binding.tvTitle.text = item.title
            binding.tvSeriesContentCount.text = "총 ${item.numberOfContent}화"
            binding.tvNew.visibility = if (item.isNew) {
                View.VISIBLE
            } else {
                View.GONE
            }

            binding.tvPopular.visibility = if (item.isPopular) {
                View.VISIBLE
            } else {
                View.GONE
            }

            if (item.isComplete) {
                binding.tvNew.visibility = View.GONE
                binding.tvComplete.visibility = View.VISIBLE
            } else {
                binding.tvComplete.visibility = View.GONE
            }

            binding.root.setOnClickListener { onClickItem(item.seriesId) }
        }
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int) = ViewHolder(
        parent.context,
        ItemSeriesOriginalAudioDramaBinding.inflate(
            LayoutInflater.from(parent.context),
            parent,
            false
        )
    )

    override fun onBindViewHolder(holder: ViewHolder, position: Int) {
        holder.bind(items[position])
    }

    override fun getItemCount() = items.count()

    @SuppressLint("NotifyDataSetChanged")
    fun addItems(items: List<GetSeriesListResponse.SeriesListItem>) {
        this.items.addAll(items)
        notifyDataSetChanged()
    }

    fun isVisibleRecyclerView(): Boolean {
        return items.isNotEmpty()
    }
}
