package kr.co.vividnext.sodalive.audio_content.order

import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import com.orhanobut.logger.Logger
import io.reactivex.rxjava3.android.schedulers.AndroidSchedulers
import io.reactivex.rxjava3.schedulers.Schedulers
import kr.co.vividnext.sodalive.audio_content.AudioContentRepository
import kr.co.vividnext.sodalive.base.BaseViewModel
import kr.co.vividnext.sodalive.common.SharedPreferenceManager

class AudioContentOrderListViewModel(
    private val repository: AudioContentRepository
) : BaseViewModel() {

    private val _toastLiveData = MutableLiveData<String?>()
    val toastLiveData: LiveData<String?>
        get() = _toastLiveData

    private var _isLoading = MutableLiveData(false)
    val isLoading: LiveData<Boolean>
        get() = _isLoading

    private var _orderList = MutableLiveData<List<GetAudioContentOrderListItem>>()
    val orderList: LiveData<List<GetAudioContentOrderListItem>>
        get() = _orderList

    private var _totalCount = MutableLiveData<Int>()
    val totalCount: LiveData<Int>
        get() = _totalCount

    private var isLast = false
    var page = 1
    private val size = 10

    fun getAudioContentOrderList(onFailure: (() -> Unit)? = null) {
        if (_isLoading.value == false) {
            _isLoading.value = true
            compositeDisposable.add(
                repository.getAudioContentOrderList(
                    page = page,
                    size = size,
                    token = "Bearer ${SharedPreferenceManager.token}"
                )
                    .subscribeOn(Schedulers.io())
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe(
                        {
                            if (it.success && it.data != null) {
                                _totalCount.value = it.data.totalCount
                                page += 1
                                if (it.data.items.isNotEmpty()) {
                                    _orderList.postValue(it.data.items)
                                } else {
                                    _orderList.postValue(listOf())
                                    isLast = true
                                }
                            } else {
                                if (it.message != null) {
                                    _toastLiveData.postValue(it.message)
                                } else {
                                    _toastLiveData.postValue(
                                        "알 수 없는 오류가 발생했습니다. 다시 시도해 주세요."
                                    )
                                }

                                if (onFailure != null) {
                                    onFailure()
                                }
                            }
                            _isLoading.value = false
                        },
                        {
                            _isLoading.value = false
                            it.message?.let { message -> Logger.e(message) }
                            _toastLiveData.postValue("알 수 없는 오류가 발생했습니다. 다시 시도해 주세요.")
                            if (onFailure != null) {
                                onFailure()
                            }
                        }
                    )
            )
        }
    }
}
