package kr.co.vividnext.sodalive.audio_content.playlist

import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import com.orhanobut.logger.Logger
import io.reactivex.rxjava3.android.schedulers.AndroidSchedulers
import io.reactivex.rxjava3.schedulers.Schedulers
import kr.co.vividnext.sodalive.base.BaseViewModel
import kr.co.vividnext.sodalive.common.SharedPreferenceManager

class AudioContentPlaylistListViewModel(
    private val repository: AudioContentPlaylistRepository
) : BaseViewModel() {
    private val _toastLiveData = MutableLiveData<String?>()
    val toastLiveData: LiveData<String?>
        get() = _toastLiveData

    private var _isLoading = MutableLiveData(false)
    val isLoading: LiveData<Boolean>
        get() = _isLoading

    private val _totalCountLiveData = MutableLiveData<Int>()
    val totalCountLiveData: LiveData<Int>
        get() = _totalCountLiveData

    private val _playlistLiveData = MutableLiveData<List<GetPlaylistsItem>>()
    val playlistLiveData: LiveData<List<GetPlaylistsItem>>
        get() = _playlistLiveData

    fun getPlaylistList() {
        _isLoading.value = true
        compositeDisposable.add(
            repository.getPlaylistList(token = "Bearer ${SharedPreferenceManager.token}")
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(
                    {
                        _isLoading.value = false
                        if (it.success && it.data != null) {
                            _totalCountLiveData.value = it.data.totalCount
                            _playlistLiveData.value = it.data.items
                        } else {
                            if (it.message != null) {
                                _toastLiveData.postValue(it.message)
                            } else {
                                _toastLiveData.postValue(
                                    "알 수 없는 오류가 발생했습니다. 다시 시도해 주세요."
                                )
                            }
                        }
                    },
                    {
                        _isLoading.value = false
                        it.message?.let { message -> Logger.e(message) }
                        _toastLiveData.postValue("알 수 없는 오류가 발생했습니다. 다시 시도해 주세요.")
                    }
                )
        )
    }
}
