package kr.co.vividnext.sodalive.audio_content.series

import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import com.google.gson.annotations.SerializedName
import com.orhanobut.logger.Logger
import io.reactivex.rxjava3.android.schedulers.AndroidSchedulers
import io.reactivex.rxjava3.schedulers.Schedulers
import kr.co.vividnext.sodalive.base.BaseViewModel
import kr.co.vividnext.sodalive.common.SharedPreferenceManager

class SeriesListAllViewModel(private val repository: SeriesRepository) : BaseViewModel() {

    enum class SeriesSortType {
        @SerializedName("NEWEST") NEWEST,
        @SerializedName("OLDEST") OLDEST
    }

    private val _toastLiveData = MutableLiveData<String?>()
    val toastLiveData: LiveData<String?>
        get() = _toastLiveData

    private var _isLoading = MutableLiveData(false)
    val isLoading: LiveData<Boolean>
        get() = _isLoading

    private var _seriesListLiveData = MutableLiveData<List<GetSeriesListResponse.SeriesListItem>>()
    val seriesListLiveData: LiveData<List<GetSeriesListResponse.SeriesListItem>>
        get() = _seriesListLiveData

    var creatorId = 0L
    var isLast = false
    var page = 1
    private val size = 10

    fun getSeriesList() {
        if (!_isLoading.value!! && !isLast) {
            _isLoading.value = true

            compositeDisposable.add(
                repository.getSeriesList(
                    creatorId = creatorId,
                    sortType = SeriesSortType.NEWEST,
                    page = page,
                    size = size,
                    token = "Bearer ${SharedPreferenceManager.token}"
                )
                    .subscribeOn(Schedulers.io())
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe(
                        {
                            if (it.success && it.data != null) {
                                page += 1

                                if (it.data.items.isNotEmpty()) {
                                    _seriesListLiveData.value = it.data.items
                                } else {
                                    isLast = true
                                }
                            } else {
                                if (it.message != null) {
                                    _toastLiveData.value = it.message
                                } else {
                                    _toastLiveData.value = "알 수 없는 오류가 발생했습니다. 다시 시도해 주세요."

                                }
                            }

                            _isLoading.value = false
                        },
                        {
                            _isLoading.value = false
                            it.message?.let { message -> Logger.e(message) }
                            _toastLiveData.postValue("알 수 없는 오류가 발생했습니다. 다시 시도해 주세요.")
                        }
                    )
            )
        }
    }
}
