package kr.co.vividnext.sodalive.audition

import io.reactivex.rxjava3.core.Single
import kr.co.vividnext.sodalive.audition.applicant.GetAuditionApplicantListResponse
import kr.co.vividnext.sodalive.audition.detail.GetAuditionDetailResponse
import kr.co.vividnext.sodalive.audition.role.AuditionRoleDetailViewModel
import kr.co.vividnext.sodalive.audition.role.GetAuditionRoleDetailResponse
import kr.co.vividnext.sodalive.audition.vote.VoteAuditionApplicantRequest
import kr.co.vividnext.sodalive.common.ApiResponse
import okhttp3.MultipartBody
import okhttp3.RequestBody
import retrofit2.http.Body
import retrofit2.http.GET
import retrofit2.http.Header
import retrofit2.http.Multipart
import retrofit2.http.POST
import retrofit2.http.Part
import retrofit2.http.Path
import retrofit2.http.Query

interface AuditionApi {
    @GET("/audition")
    fun getAuditionList(
        @Query("page") page: Int,
        @Query("size") size: Int,
        @Header("Authorization") authHeader: String
    ): Single<ApiResponse<GetAuditionListResponse>>

    @GET("/audition/{id}")
    fun getAuditionDetail(
        @Path("id") id: Long,
        @Header("Authorization") authHeader: String
    ): Single<ApiResponse<GetAuditionDetailResponse>>

    @GET("/audition/role/{id}")
    fun getAuditionRoleDetail(
        @Path("id") id: Long,
        @Header("Authorization") authHeader: String
    ): Single<ApiResponse<GetAuditionRoleDetailResponse>>

    @GET("/audition/applicant")
    fun getAuditionApplicantList(
        @Query("auditionRoleId") auditionRoleId: Long,
        @Query("sortType") sortType: AuditionRoleDetailViewModel.AuditionApplicantSortType,
        @Query("page") page: Int,
        @Query("size") size: Int,
        @Header("Authorization") authHeader: String
    ): Single<ApiResponse<GetAuditionApplicantListResponse>>

    @POST("/audition/applicant")
    @Multipart
    fun applyAudition(
        @Part contentFile: MultipartBody.Part,
        @Part("request") request: RequestBody,
        @Header("Authorization") authHeader: String
    ): Single<ApiResponse<Any>>

    @POST("/audition/vote")
    fun voteApplicant(
        @Body request: VoteAuditionApplicantRequest,
        @Header("Authorization") authHeader: String
    ): Single<ApiResponse<Any>>
}
