package kr.co.vividnext.sodalive.audition

import android.content.Intent
import android.graphics.Rect
import android.net.Uri
import android.os.Bundle
import android.view.View
import android.widget.Toast
import androidx.media3.common.util.UnstableApi
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import kr.co.vividnext.sodalive.R
import kr.co.vividnext.sodalive.audition.detail.AuditionDetailActivity
import kr.co.vividnext.sodalive.base.BaseFragment
import kr.co.vividnext.sodalive.common.Constants
import kr.co.vividnext.sodalive.common.LoadingDialog
import kr.co.vividnext.sodalive.common.SharedPreferenceManager
import kr.co.vividnext.sodalive.databinding.FragmentAuditionBinding
import kr.co.vividnext.sodalive.extensions.dpToPx
import org.koin.android.ext.android.inject

@UnstableApi
class AuditionFragment : BaseFragment<FragmentAuditionBinding>(
    FragmentAuditionBinding::inflate
) {
    private val viewModel: AuditionViewModel by inject()

    private lateinit var adapter: AuditionListAdapter
    private lateinit var loadingDialog: LoadingDialog

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        setupView()
        bindData()

        viewModel.getAuditionList()
    }

    override fun onResume() {
        super.onResume()

        setAuditionNotification(SharedPreferenceManager.isAuditionNotification)
    }

    private fun setupView() {
        loadingDialog = LoadingDialog(requireActivity(), layoutInflater)

        val recyclerView = binding.rvAudition
        adapter = AuditionListAdapter {
            startActivity(
                Intent(requireContext(), AuditionDetailActivity::class.java).apply {
                    putExtra(Constants.EXTRA_AUDITION_ID, it)
                }
            )
        }

        recyclerView.layoutManager = LinearLayoutManager(
            requireContext(),
            LinearLayoutManager.VERTICAL,
            false
        )

        recyclerView.addItemDecoration(object : RecyclerView.ItemDecoration() {
            override fun getItemOffsets(
                outRect: Rect,
                view: View,
                parent: RecyclerView,
                state: RecyclerView.State
            ) {
                super.getItemOffsets(outRect, view, parent, state)

                when (parent.getChildAdapterPosition(view)) {
                    0 -> {
                        outRect.top = 0
                        outRect.bottom = 12.5f.dpToPx().toInt()
                    }

                    adapter.itemCount - 1 -> {
                        outRect.top = 12.5f.dpToPx().toInt()
                        outRect.bottom = 0
                    }

                    else -> {
                        outRect.top = 12.5f.dpToPx().toInt()
                        outRect.bottom = 12.5f.dpToPx().toInt()
                    }
                }
            }
        })

        recyclerView.adapter = adapter

        binding.ivNotification.setOnClickListener {
            viewModel.updateNotificationSettings {
                setAuditionNotification(it)
            }
        }

        binding.rlHowToUse.setOnClickListener {
            startActivity(
                Intent(
                    Intent.ACTION_VIEW,
                    Uri.parse("https://bit.ly/40EYuJc")
                )
            )
        }
    }

    private fun bindData() {
        viewModel.toastLiveData.observe(viewLifecycleOwner) {
            it?.let { Toast.makeText(requireActivity(), it, Toast.LENGTH_LONG).show() }
        }

        viewModel.isLoading.observe(viewLifecycleOwner) {
            if (it) {
                loadingDialog.show(screenWidth, "")
            } else {
                loadingDialog.dismiss()
            }
        }

        viewModel.auditionListLiveData.observe(viewLifecycleOwner) {
            adapter.addGroupedList(it)
        }
    }

    private fun setAuditionNotification(isAuditionNotification: Boolean) {
        binding.ivNotification.setImageResource(
            if (isAuditionNotification) {
                R.drawable.btn_audition_notification_selected
            } else {
                R.drawable.btn_audition_notification_normal
            }
        )
    }
}
