package kr.co.vividnext.sodalive.audition

import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import com.orhanobut.logger.Logger
import io.reactivex.rxjava3.android.schedulers.AndroidSchedulers
import io.reactivex.rxjava3.schedulers.Schedulers
import kr.co.vividnext.sodalive.base.BaseViewModel
import kr.co.vividnext.sodalive.common.SharedPreferenceManager
import kr.co.vividnext.sodalive.settings.notification.UpdateNotificationSettingRequest
import kr.co.vividnext.sodalive.user.UserRepository

class AuditionViewModel(
    private val repository: AuditionRepository,
    private val userRepository: UserRepository
) : BaseViewModel() {
    private val _toastLiveData = MutableLiveData<String?>()
    val toastLiveData: LiveData<String?>
        get() = _toastLiveData

    private var _isLoading = MutableLiveData(false)
    val isLoading: LiveData<Boolean>
        get() = _isLoading

    private val _auditionListLiveData = MutableLiveData<GetAuditionListResponse>()
    val auditionListLiveData: LiveData<GetAuditionListResponse>
        get() = _auditionListLiveData

    var page = 1
    var isLast = false
    private val pageSize = 10

    fun getAuditionList() {
        if (!isLast && !_isLoading.value!!) {
            _isLoading.value = true
            compositeDisposable.add(
                repository.getAuditionList(
                    page = page,
                    size = pageSize,
                    token = "Bearer ${SharedPreferenceManager.token}"
                )
                    .subscribeOn(Schedulers.io())
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe(
                        {
                            _isLoading.value = false
                            if (it.success && it.data != null) {
                                _auditionListLiveData.value = it.data!!
                                if (it.data.items.isNotEmpty()) {
                                    page += 1
                                } else {
                                    isLast = true
                                }
                            } else {
                                if (it.message != null) {
                                    _toastLiveData.postValue(it.message)
                                } else {
                                    _toastLiveData.postValue(
                                        "알 수 없는 오류가 발생했습니다. 다시 시도해 주세요."
                                    )
                                }
                            }
                        },
                        {
                            _isLoading.postValue(false)
                            it.message?.let { message -> Logger.e(message) }
                            _toastLiveData.postValue("알 수 없는 오류가 발생했습니다. 다시 시도해 주세요.")
                        }
                    )
            )
        }
    }

    fun updateNotificationSettings(onSuccess: (Boolean) -> Unit) {
        val isActiveNotification = !SharedPreferenceManager.isAuditionNotification

        compositeDisposable.add(
            userRepository.updateNotificationSettings(
                request = UpdateNotificationSettingRequest(
                    audition = isActiveNotification
                ),
                token = "Bearer ${SharedPreferenceManager.token}"
            )
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(
                    {
                        if (it.success) {
                            SharedPreferenceManager.isAuditionNotification = isActiveNotification
                            onSuccess(isActiveNotification)
                        } else {
                            onSuccess(!isActiveNotification)
                        }
                    },
                    {}
                )
        )
    }
}
