package kr.co.vividnext.sodalive.audition.applicant

import android.app.Dialog
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.FrameLayout
import android.widget.Toast
import com.google.android.material.bottomsheet.BottomSheetBehavior
import com.google.android.material.bottomsheet.BottomSheetDialog
import com.google.android.material.bottomsheet.BottomSheetDialogFragment
import kr.co.vividnext.sodalive.databinding.FragmentAuditionApplyDialogBinding

class AuditionApplyDialogFragment(
    private val fileName: String,
    private val onClickApply: (String) -> Unit,
    private val onClickClose: () -> Unit
) : BottomSheetDialogFragment() {
    private lateinit var binding: FragmentAuditionApplyDialogBinding

    override fun onCreateDialog(savedInstanceState: Bundle?): Dialog {
        val dialog = super.onCreateDialog(savedInstanceState)

        dialog.setOnShowListener {
            val d = it as BottomSheetDialog
            val bottomSheet = d.findViewById<FrameLayout>(
                com.google.android.material.R.id.design_bottom_sheet
            )
            if (bottomSheet != null) {
                val bottomSheetBehavior = BottomSheetBehavior.from(bottomSheet)
                bottomSheetBehavior.state = BottomSheetBehavior.STATE_EXPANDED
                bottomSheetBehavior.isDraggable = false
            }
        }

        return dialog
    }

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        binding = FragmentAuditionApplyDialogBinding.inflate(inflater, container, false)
        return binding.root
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        binding.ivClose.setOnClickListener {
            onClickClose()
            dismiss()
        }

        binding.tvAuditionApply.setOnClickListener {
            val phoneNumber = binding.etPhoneNumber.text.toString()
            if (phoneNumber.isBlank() || phoneNumber.length != 11) {
                Toast.makeText(
                    activity,
                    "잘못된 연락처 입니다.\n다시 입력해 주세요.",
                    Toast.LENGTH_LONG
                ).show()
            } else if (!binding.tvAgree.isSelected) {
                Toast.makeText(
                    activity,
                    "연락처 수집 및 활용에 동의하셔야 오디션 지원이 가능합니다.",
                    Toast.LENGTH_LONG
                ).show()
            } else {
                dismiss()
                onClickApply(binding.etPhoneNumber.text.toString())
            }
        }

        binding.llAgree.setOnClickListener { it.isSelected = !it.isSelected }
        binding.tvAudioFileName.text = fileName
    }
}
