package kr.co.vividnext.sodalive.audition.detail

import android.content.Intent
import android.graphics.Rect
import android.os.Bundle
import android.view.View
import android.widget.Toast
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import coil.load
import coil.transform.RoundedCornersTransformation
import com.bumptech.glide.Glide
import com.bumptech.glide.load.MultiTransformation
import com.bumptech.glide.load.resource.bitmap.CenterCrop
import com.bumptech.glide.load.resource.bitmap.RoundedCorners
import com.bumptech.glide.request.RequestOptions
import kr.co.vividnext.sodalive.R
import kr.co.vividnext.sodalive.audition.role.AuditionRoleDetailActivity
import kr.co.vividnext.sodalive.base.BaseActivity
import kr.co.vividnext.sodalive.common.Constants
import kr.co.vividnext.sodalive.common.LoadingDialog
import kr.co.vividnext.sodalive.databinding.ActivityAuditionDetailBinding
import kr.co.vividnext.sodalive.extensions.dpToPx
import org.koin.android.ext.android.inject

class AuditionDetailActivity : BaseActivity<ActivityAuditionDetailBinding>(
    ActivityAuditionDetailBinding::inflate
) {
    private val viewModel: AuditionDetailViewModel by inject()

    private lateinit var loadingDialog: LoadingDialog
    private lateinit var adapter: AuditionDetailRoleAdapter

    private var auditionId: Long = 0
    private var isOpenInformation = false

    override fun onCreate(savedInstanceState: Bundle?) {
        auditionId = intent.getLongExtra(Constants.EXTRA_AUDITION_ID, 0)

        if (auditionId <= 0) {
            Toast.makeText(
                applicationContext,
                "잘못된 요청입니다.\n다시 시도해 주세요.",
                Toast.LENGTH_LONG
            ).show()

            finish()
        }

        super.onCreate(savedInstanceState)

        bindData()
        viewModel.getAuditionDetail(auditionId = auditionId)
    }

    override fun setupView() {
        loadingDialog = LoadingDialog(this, layoutInflater)
        binding.toolbar.tvBack.setOnClickListener { finish() }
        binding.tvOpen.setOnClickListener {
            isOpenInformation = !isOpenInformation
            if (isOpenInformation) {
                binding.tvInformation.maxLines = Int.MAX_VALUE
                binding.tvOpen.text = "접기"
                binding.tvOpen.setCompoundDrawablesWithIntrinsicBounds(
                    R.drawable.ic_live_detail_top,
                    0,
                    0,
                    0
                )
            } else {
                binding.tvInformation.maxLines = 3
                binding.tvOpen.text = "펼치기"
                binding.tvOpen.setCompoundDrawablesWithIntrinsicBounds(
                    R.drawable.ic_live_detail_bottom,
                    0,
                    0,
                    0
                )
            }
        }

        adapter = AuditionDetailRoleAdapter {
            startActivity(
                Intent(applicationContext, AuditionRoleDetailActivity::class.java).apply {
                    putExtra(Constants.EXTRA_AUDITION_ROLE_ID, it)
                }
            )
        }

        binding.rvRole.layoutManager = LinearLayoutManager(
            applicationContext,
            LinearLayoutManager.VERTICAL,
            false
        )

        binding.rvRole.addItemDecoration(object : RecyclerView.ItemDecoration() {
            override fun getItemOffsets(
                outRect: Rect,
                view: View,
                parent: RecyclerView,
                state: RecyclerView.State
            ) {
                super.getItemOffsets(outRect, view, parent, state)

                when (parent.getChildAdapterPosition(view)) {
                    0 -> {
                        outRect.top = 0
                        outRect.bottom = 12.5f.dpToPx().toInt()
                    }

                    adapter.itemCount - 1 -> {
                        outRect.top = 12.5f.dpToPx().toInt()
                        outRect.bottom = 0
                    }

                    else -> {
                        outRect.top = 12.5f.dpToPx().toInt()
                        outRect.bottom = 12.5f.dpToPx().toInt()
                    }
                }
            }
        })

        binding.rvRole.adapter = adapter
    }

    private fun bindData() {
        viewModel.toastLiveData.observe(this) {
            it?.let { Toast.makeText(applicationContext, it, Toast.LENGTH_LONG).show() }
        }

        viewModel.isLoading.observe(this) {
            if (it) {
                loadingDialog.show(screenWidth, "")
            } else {
                loadingDialog.dismiss()
            }
        }

        viewModel.auditionDetailLiveData.observe(this) {
            Glide
                .with(this)
                .load(it.imageUrl)
                .apply(
                    RequestOptions().transform(
                        MultiTransformation(
                            CenterCrop(),
                            RoundedCorners(5f.dpToPx().toInt())
                        )
                    )
                )
                .placeholder(R.drawable.bg_black)
                .into(binding.ivCover)

            binding.toolbar.tvBack.text = it.title
            binding.tvInformation.text = it.information
            adapter.addItems(it.roleList)
        }
    }
}
