package kr.co.vividnext.sodalive.audition.detail

import android.content.Context
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.recyclerview.widget.DiffUtil
import androidx.recyclerview.widget.ListAdapter
import androidx.recyclerview.widget.RecyclerView
import com.bumptech.glide.Glide
import com.bumptech.glide.load.MultiTransformation
import com.bumptech.glide.load.resource.bitmap.CenterCrop
import com.bumptech.glide.load.resource.bitmap.RoundedCorners
import com.bumptech.glide.request.RequestOptions
import kr.co.vividnext.sodalive.R
import kr.co.vividnext.sodalive.databinding.ItemAuditionRoleBinding
import kr.co.vividnext.sodalive.extensions.dpToPx

class AuditionDetailRoleAdapter(
    private val onItemClick: (Long) -> Unit
) : ListAdapter<GetAuditionRoleListData, AuditionDetailRoleAdapter.ViewHolder>(DiffCallback()) {
    private var items = mutableListOf<GetAuditionRoleListData>()

    class DiffCallback : DiffUtil.ItemCallback<GetAuditionRoleListData>() {
        override fun areItemsTheSame(
            oldItem: GetAuditionRoleListData,
            newItem: GetAuditionRoleListData
        ): Boolean {
            return oldItem == newItem
        }

        override fun areContentsTheSame(
            oldItem: GetAuditionRoleListData,
            newItem: GetAuditionRoleListData
        ): Boolean {
            return oldItem.roleId == newItem.roleId
        }
    }

    inner class ViewHolder(
        private val context: Context,
        private val binding: ItemAuditionRoleBinding
    ) : RecyclerView.ViewHolder(binding.root) {
        fun bind(item: GetAuditionRoleListData) {
            Glide
                .with(context)
                .load(item.imageUrl)
                .apply(
                    RequestOptions().transform(
                        MultiTransformation(
                            CenterCrop(),
                            RoundedCorners(6.7f.dpToPx().toInt())
                        )
                    )
                )
                .placeholder(R.drawable.bg_black)
                .into(binding.ivCover)

            binding.tvName.text = item.name

            if (item.isComplete) {
                binding.blackCover.visibility = View.VISIBLE
                binding.tvStatus.text = "모집완료"
                binding.tvStatus.setBackgroundResource(R.drawable.bg_round_corner_13_3_909090)
            } else {
                binding.blackCover.visibility = View.GONE
                binding.tvStatus.text = "모집중"
                binding.tvStatus.setBackgroundResource(R.drawable.bg_round_corner_13_3_3bb9f1)
            }

            binding.root.setOnClickListener {
                if (!item.isComplete) {
                    onItemClick(item.roleId)
                }
            }
        }
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int) = ViewHolder(
        parent.context,
        ItemAuditionRoleBinding.inflate(
            LayoutInflater.from(parent.context),
            parent,
            false
        )
    )

    override fun onBindViewHolder(holder: ViewHolder, position: Int) {
        holder.bind(items[position])
    }

    fun addItems(items: List<GetAuditionRoleListData>) {
        this.items.addAll(items)
        submitList(this.items.toList())
    }
}
