package kr.co.vividnext.sodalive.audition.role

import android.content.Intent
import android.graphics.Rect
import android.net.Uri
import android.os.Bundle
import android.provider.OpenableColumns
import android.view.Gravity
import android.view.View
import android.widget.Toast
import androidx.activity.result.contract.ActivityResultContracts
import androidx.core.content.ContextCompat
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.bumptech.glide.Glide
import com.bumptech.glide.load.MultiTransformation
import com.bumptech.glide.load.resource.bitmap.CenterCrop
import com.bumptech.glide.load.resource.bitmap.RoundedCorners
import com.bumptech.glide.request.RequestOptions
import kr.co.vividnext.sodalive.R
import kr.co.vividnext.sodalive.audition.applicant.ApplicationMethodDialog
import kr.co.vividnext.sodalive.audition.applicant.AuditionApplicantListAdapter
import kr.co.vividnext.sodalive.audition.applicant.AuditionApplicantMediaPlayerManager
import kr.co.vividnext.sodalive.audition.applicant.AuditionApplyDialogFragment
import kr.co.vividnext.sodalive.base.BaseActivity
import kr.co.vividnext.sodalive.base.SodaDialog
import kr.co.vividnext.sodalive.common.Constants
import kr.co.vividnext.sodalive.common.LoadingDialog
import kr.co.vividnext.sodalive.common.RealPathUtil
import kr.co.vividnext.sodalive.common.SharedPreferenceManager
import kr.co.vividnext.sodalive.databinding.ActivityAuditionRoleDetailBinding
import kr.co.vividnext.sodalive.explorer.profile.UserProfileActivity
import kr.co.vividnext.sodalive.explorer.profile.creator_community.write.RecordingVoiceFragment
import kr.co.vividnext.sodalive.extensions.dpToPx
import org.koin.android.ext.android.inject
import java.io.File

class AuditionRoleDetailActivity : BaseActivity<ActivityAuditionRoleDetailBinding>(
    ActivityAuditionRoleDetailBinding::inflate
), RecordingVoiceFragment.OnAudioRecordedListener {
    private val viewModel: AuditionRoleDetailViewModel by inject()

    private lateinit var loadingDialog: LoadingDialog
    private lateinit var adapter: AuditionApplicantListAdapter

    private var auditionRoleId: Long = 0
    private var isOpenInformation = false
    private var reApplication = false
    private var isDeleteAudioFile = false
    private var isShowNotifyVote = true

    private val selectAudioActivityResultLauncher = registerForActivityResult(
        ActivityResultContracts.StartActivityForResult()
    ) { result ->
        val resultCode = result.resultCode
        val data = result.data

        if (resultCode == RESULT_OK) {
            val fileUri = data?.data
            if (fileUri != null) {
                viewModel.audioFileName = getFileName(fileUri).toString()
                viewModel.audioFile = File(
                    RealPathUtil.getRealPath(applicationContext, fileUri!!)
                )
                showAuditionApplyDialog()
            } else {
                Toast.makeText(
                    this,
                    "잘못된 녹음 파일 입니다.\n다시 선택해 주세요.",
                    Toast.LENGTH_SHORT
                ).show()
            }
        } else {
            Toast.makeText(
                this,
                "잘못된 녹음 파일 입니다.\n다시 선택해 주세요.",
                Toast.LENGTH_SHORT
            ).show()
        }
    }

    private lateinit var mediaPlayerManager: AuditionApplicantMediaPlayerManager

    override fun onCreate(savedInstanceState: Bundle?) {
        auditionRoleId = intent.getLongExtra(Constants.EXTRA_AUDITION_ROLE_ID, 0)

        if (auditionRoleId <= 0) {
            Toast.makeText(
                applicationContext,
                "잘못된 요청입니다.\n다시 시도해 주세요.",
                Toast.LENGTH_LONG
            ).show()

            finish()
        }

        super.onCreate(savedInstanceState)

        mediaPlayerManager = AuditionApplicantMediaPlayerManager(
            context = this,
            updatePlayPauseState = { position, isPlaying ->
                adapter.updatePlayingIndex(position, isPlaying)
            },
            updateTotalDuration = { duration ->
                adapter.updateTotalDuration(totalDuration = duration)
            },
            updateCurrentTime = { currentTime ->
                adapter.updateCurrentTime(currentTime)
            },
            showLoadingDialog = {
                if (it) {
                    loadingDialog.show(screenWidth, "")
                } else {
                    loadingDialog.dismiss()
                }
            }
        )

        bindData()
        viewModel.getAuditionRoleDetail(auditionRoleId = auditionRoleId)
    }

    override fun onPause() {
        mediaPlayerManager.pauseContent()
        super.onPause()
    }

    override fun onDestroy() {
        mediaPlayerManager.stopContent()
        super.onDestroy()
    }

    override fun setupView() {
        loadingDialog = LoadingDialog(this, layoutInflater)
        binding.toolbar.tvBack.setOnClickListener { finish() }
        binding.tvOpen.setOnClickListener {
            isOpenInformation = !isOpenInformation
            if (isOpenInformation) {
                binding.tvInformation.maxLines = Int.MAX_VALUE
                binding.tvOpen.text = "접기"
                binding.tvOpen.setCompoundDrawablesWithIntrinsicBounds(
                    R.drawable.ic_live_detail_top,
                    0,
                    0,
                    0
                )
            } else {
                binding.tvInformation.maxLines = 3
                binding.tvOpen.text = "펼치기"
                binding.tvOpen.setCompoundDrawablesWithIntrinsicBounds(
                    R.drawable.ic_live_detail_bottom,
                    0,
                    0,
                    0
                )
            }
        }

        binding.tvApplicant.setOnClickListener {
            if (SharedPreferenceManager.isAuth) {
                if (reApplication) {
                    SodaDialog(
                        activity = this@AuditionRoleDetailActivity,
                        layoutInflater = layoutInflater,
                        title = "재지원 안내",
                        desc = "재지원 시 이전 지원 내역은 삭제되며 받은 투표수는 무효 처리됩니다.",
                        confirmButtonTitle = "확인",
                        confirmButtonClick = { showApplicationMethodDialog() }
                    ).show(screenWidth)
                } else {
                    showApplicationMethodDialog()
                }
            } else {
                SodaDialog(
                    activity = this@AuditionRoleDetailActivity,
                    layoutInflater = layoutInflater,
                    title = "- 본인인증 -",
                    desc = "마이페이지에서 '본인인증'을 하고 다시 오디션에 지원해 주세요.",
                    confirmButtonTitle = "확인",
                    confirmButtonClick = {}
                ).show(screenWidth)
            }
        }

        binding.tvSortNewest.setOnClickListener {
            mediaPlayerManager.stopContent()
            viewModel.setSortType(AuditionRoleDetailViewModel.AuditionApplicantSortType.NEWEST)
        }

        binding.tvSortLikes.setOnClickListener {
            mediaPlayerManager.stopContent()
            viewModel.setSortType(AuditionRoleDetailViewModel.AuditionApplicantSortType.LIKES)
        }

        adapter = AuditionApplicantListAdapter(
            onClickPlayOrPause = { position, applicantId, voiceUrl ->
                mediaPlayerManager.toggleContent(position, applicantId, voiceUrl)
            },
            onClickNickname = { memberId ->
                startActivity(
                    Intent(applicationContext, UserProfileActivity::class.java).apply {
                        putExtra(Constants.EXTRA_USER_ID, memberId)
                    }
                )
            },
            onClickVote = { applicantId ->
                viewModel.voteApplicant(
                    applicantId,
                    onSuccess = {
                        if (isShowNotifyVote) {
                            SodaDialog(
                                activity = this@AuditionRoleDetailActivity,
                                layoutInflater = layoutInflater,
                                "[오디션 응원]",
                                "오디션을 응원하셨습니다\n(무료응원 : 1계정당 1일 1회)\n1캔으로 추가 응원을 해보세요.",
                                confirmButtonTitle = "확인",
                                confirmButtonClick = {
                                    isShowNotifyVote = false
                                },
                                descGravity = Gravity.CENTER
                            ).show(screenWidth)
                        }
                    },
                    onFailure = {
                        SodaDialog(
                            activity = this@AuditionRoleDetailActivity,
                            layoutInflater = layoutInflater,
                            "[오늘 응원 제한]",
                            "오늘 응원은 여기까지!\n하루 최대 10회까지 이용이 가능합니다.\n내일 다시 이용해주세요.",
                            confirmButtonTitle = "확인",
                            confirmButtonClick = {},
                            descGravity = Gravity.CENTER
                        ).show(screenWidth)
                    }
                )
            }
        )

        val recyclerView = binding.rvApplicant
        recyclerView.itemAnimator = null
        recyclerView.layoutManager = LinearLayoutManager(
            applicationContext,
            LinearLayoutManager.VERTICAL,
            false
        )

        recyclerView.addItemDecoration(object : RecyclerView.ItemDecoration() {
            override fun getItemOffsets(
                outRect: Rect,
                view: View,
                parent: RecyclerView,
                state: RecyclerView.State
            ) {
                super.getItemOffsets(outRect, view, parent, state)

                when (parent.getChildAdapterPosition(view)) {
                    0 -> {
                        outRect.top = 0
                        outRect.bottom = 2.7f.dpToPx().toInt()
                    }

                    adapter.itemCount - 1 -> {
                        outRect.top = 2.7f.dpToPx().toInt()
                        outRect.bottom = 33f.dpToPx().toInt()
                    }

                    else -> {
                        outRect.top = 2.7f.dpToPx().toInt()
                        outRect.bottom = 2.7f.dpToPx().toInt()
                    }
                }
            }
        })

        recyclerView.addOnScrollListener(object : RecyclerView.OnScrollListener() {
            override fun onScrolled(recyclerView: RecyclerView, dx: Int, dy: Int) {
                super.onScrolled(recyclerView, dx, dy)

                val lastVisibleItemPosition = (recyclerView.layoutManager as LinearLayoutManager?)!!
                    .findLastCompletelyVisibleItemPosition()
                val itemTotalCount = recyclerView.adapter!!.itemCount - 1

                // 스크롤이 끝에 도달했는지 확인
                if (!recyclerView.canScrollVertically(1) &&
                    lastVisibleItemPosition == itemTotalCount
                ) {
                    viewModel.getAuditionApplicantList()
                }
            }
        })

        recyclerView.adapter = adapter
    }

    private fun showApplicationMethodDialog() {
        isDeleteAudioFile = false
        ApplicationMethodDialog(
            activity = this@AuditionRoleDetailActivity,
            layoutInflater = layoutInflater,
            onClickFileUpload = {
                isDeleteAudioFile = false
                val intent = Intent().apply {
                    type = "audio/*"
                    action = Intent.ACTION_GET_CONTENT
                    addCategory(Intent.CATEGORY_OPENABLE)
                }
                selectAudioActivityResultLauncher.launch(
                    Intent.createChooser(
                        intent,
                        "Select Audio"
                    )
                )
            },
            onClickRecord = {
                isDeleteAudioFile = true
                val fragment = RecordingVoiceFragment()
                fragment.isCancelable = false
                fragment.show(supportFragmentManager, fragment.tag)
            }
        ).show(screenWidth)
    }

    private fun showAuditionApplyDialog() {
        val auditionApplyDialogFragment = AuditionApplyDialogFragment(
            fileName = viewModel.audioFileName,
            onClickApply = {
                viewModel.applyAudition(auditionRoleId = auditionRoleId, phoneNumber = it) {
                    if (isDeleteAudioFile) {
                        deleteAudioFile()
                    }
                }
            },
            onClickClose = {
                if (isDeleteAudioFile) {
                    deleteAudioFile()
                }
            }
        )
        auditionApplyDialogFragment.show(supportFragmentManager, auditionApplyDialogFragment.tag)
    }

    private fun bindData() {
        viewModel.toastLiveData.observe(this) {
            it?.let { Toast.makeText(applicationContext, it, Toast.LENGTH_LONG).show() }
        }

        viewModel.isLoading.observe(this) {
            if (it) {
                loadingDialog.show(screenWidth, "")
            } else {
                loadingDialog.dismiss()
            }
        }

        viewModel.auditionRoleDetailLiveData.observe(this) { roleDetail ->
            binding.toolbar.tvBack.text = roleDetail.name

            binding.tvAuditionScript.setOnClickListener {
                startActivity(Intent(Intent.ACTION_VIEW, Uri.parse(roleDetail.auditionScriptUrl)))
            }

            binding.tvOriginalWork.setOnClickListener {
                startActivity(Intent(Intent.ACTION_VIEW, Uri.parse(roleDetail.originalWorkUrl)))
            }

            binding.tvInformation.text = roleDetail.information

            Glide
                .with(this)
                .load(roleDetail.imageUrl)
                .apply(
                    RequestOptions().transform(
                        MultiTransformation(
                            CenterCrop(),
                            RoundedCorners(6.7f.dpToPx().toInt())
                        )
                    )
                )
                .placeholder(R.drawable.bg_black)
                .into(binding.ivCover)

            if (roleDetail.isAlreadyApplicant) {
                reApplication = true
                binding.tvApplicant.text = "오디션 재지원"
            } else {
                reApplication = false
                binding.tvApplicant.text = "오디션 지원"
            }
        }

        viewModel.totalCountLiveData.observe(this) {
            binding.groupApplicant.visibility = View.GONE
            binding.groupNoApplicant.visibility = View.GONE

            if (it > 0) {
                binding.groupApplicant.visibility = View.VISIBLE
                binding.tvApplicantCount.text = "$it"
            } else {
                binding.groupNoApplicant.visibility = View.VISIBLE
            }
        }

        viewModel.applicantListLiveData.observe(this) {
            adapter.submitList(it)
        }

        viewModel.sortTypeLiveData.observe(this) {
            binding.tvSortNewest.setTextColor(
                ContextCompat.getColor(applicationContext, R.color.color_bbbbbb)
            )
            binding.tvSortLikes.setTextColor(
                ContextCompat.getColor(applicationContext, R.color.color_bbbbbb)
            )

            if (it == AuditionRoleDetailViewModel.AuditionApplicantSortType.NEWEST) {
                binding.tvSortNewest.setTextColor(
                    ContextCompat.getColor(applicationContext, R.color.color_3bb9f1)
                )
            } else if (it == AuditionRoleDetailViewModel.AuditionApplicantSortType.LIKES) {
                binding.tvSortLikes.setTextColor(
                    ContextCompat.getColor(applicationContext, R.color.color_3bb9f1)
                )
            }
        }
    }

    private fun deleteAudioFile() {
        if (viewModel.audioFile != null && viewModel.audioFile!!.exists()) {
            viewModel.audioFile?.delete()
            viewModel.audioFileName = ""
        }
    }

    private fun getFileName(uri: Uri): String? {
        val scheme = uri.scheme
        var fileName: String? = null

        if (scheme == "content") {
            contentResolver.query(uri, null, null, null, null)?.use { cursor ->
                val nameIndex = cursor.getColumnIndex(OpenableColumns.DISPLAY_NAME)
                if (nameIndex != -1 && cursor.moveToFirst()) {
                    fileName = cursor.getString(nameIndex)
                }
            }
        } else if (scheme == "file") {
            val file = File(uri.path ?: "")
            fileName = file.name
        }

        return fileName
    }

    override fun onAudioRecorded(file: File) {
        deleteAudioFile()
        viewModel.audioFile = file
        viewModel.audioFileName = file.name
        showAuditionApplyDialog()
    }
}
