package kr.co.vividnext.sodalive.audition.role

import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import com.google.gson.Gson
import com.google.gson.annotations.SerializedName
import com.orhanobut.logger.Logger
import io.reactivex.rxjava3.android.schedulers.AndroidSchedulers
import io.reactivex.rxjava3.core.Observable
import io.reactivex.rxjava3.schedulers.Schedulers
import kr.co.vividnext.sodalive.audition.AuditionRepository
import kr.co.vividnext.sodalive.audition.applicant.ApplyAuditionRoleRequest
import kr.co.vividnext.sodalive.audition.applicant.GetAuditionRoleApplicantItem
import kr.co.vividnext.sodalive.audition.vote.VoteAuditionApplicantRequest
import kr.co.vividnext.sodalive.base.BaseViewModel
import kr.co.vividnext.sodalive.common.SharedPreferenceManager
import okhttp3.MediaType
import okhttp3.MediaType.Companion.toMediaType
import okhttp3.MultipartBody
import okhttp3.RequestBody
import okhttp3.RequestBody.Companion.toRequestBody
import okio.BufferedSink
import java.io.File
import java.util.TimeZone

class AuditionRoleDetailViewModel(private val repository: AuditionRepository) : BaseViewModel() {
    private val _toastLiveData = MutableLiveData<String?>()
    val toastLiveData: LiveData<String?>
        get() = _toastLiveData

    private val _isLoading = MutableLiveData(false)
    val isLoading: LiveData<Boolean>
        get() = _isLoading

    private val _auditionRoleDetailLiveData = MutableLiveData<GetAuditionRoleDetailResponse>()
    val auditionRoleDetailLiveData: LiveData<GetAuditionRoleDetailResponse>
        get() = _auditionRoleDetailLiveData

    private val _totalCountLiveData = MutableLiveData(0)
    val totalCountLiveData: LiveData<Int>
        get() = _totalCountLiveData

    private val _applicantListLiveData = MutableLiveData<List<GetAuditionRoleApplicantItem>>()
    val applicantListLiveData: LiveData<List<GetAuditionRoleApplicantItem>>
        get() = _applicantListLiveData

    private val _sortTypeLiveData = MutableLiveData(AuditionApplicantSortType.NEWEST)
    val sortTypeLiveData: LiveData<AuditionApplicantSortType>
        get() = _sortTypeLiveData

    var page = 1
    var isLast = false
    var audioFile: File? = null
    var audioFileName: String = ""
    private var auditionRoleId = -1L
    private val pageSize = 10

    fun getAuditionRoleDetail(auditionRoleId: Long, onFailure: (() -> Unit)? = null) {
        this.auditionRoleId = auditionRoleId
        _isLoading.value = true

        val auditionRoleDetail = repository.getAuditionRoleDetail(
            auditionRoleId = auditionRoleId,
            token = "Bearer ${SharedPreferenceManager.token}"
        )

        val getApplicantList = repository.getAuditionApplicantList(
            auditionRoleId = auditionRoleId,
            sortType = _sortTypeLiveData.value!!,
            page = page,
            size = pageSize,
            token = "Bearer ${SharedPreferenceManager.token}"
        )

        compositeDisposable.add(
            Observable.combineLatest(
                auditionRoleDetail.toObservable(),
                getApplicantList.toObservable()
            ) { response1, response2 ->
                Pair(response1, response2)
            }
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(
                    { pair ->
                        val roleDetailResponse = pair.first
                        val applicantListResponse = pair.second

                        if (roleDetailResponse.success && roleDetailResponse.data != null) {
                            _auditionRoleDetailLiveData.value = roleDetailResponse.data!!
                        } else {
                            if (roleDetailResponse.message != null) {
                                _toastLiveData.value = roleDetailResponse.message
                            } else {
                                _toastLiveData.value = "알 수 없는 오류가 발생했습니다. 다시 시도해 주세요."
                            }

                            if (onFailure != null) {
                                onFailure()
                            }
                        }

                        if (applicantListResponse.success && applicantListResponse.data != null) {
                            val data = applicantListResponse.data

                            _totalCountLiveData.value = data.totalCount
                            addApplicantList(data.items)

                            if (data.items.isEmpty()) {
                                isLast = true
                            } else {
                                page += 1
                            }
                        } else {
                            if (applicantListResponse.message != null) {
                                _toastLiveData.value = applicantListResponse.message
                            } else {
                                _toastLiveData.value = "알 수 없는 오류가 발생했습니다. 다시 시도해 주세요."
                            }

                            if (onFailure != null) {
                                onFailure()
                            }
                        }

                        _isLoading.value = false
                    },
                    {
                        _isLoading.value = false
                        it.message?.let { message -> Logger.e(message) }
                        _toastLiveData.postValue("알 수 없는 오류가 발생했습니다. 다시 시도해 주세요.")
                        if (onFailure != null) {
                            onFailure()
                        }
                    }
                )
        )
    }

    fun getAuditionApplicantList() {
        if (!_isLoading.value!! && !isLast) {
            _isLoading.value = true

            compositeDisposable.add(
                repository.getAuditionApplicantList(
                    auditionRoleId = auditionRoleId,
                    sortType = _sortTypeLiveData.value!!,
                    page = page,
                    size = pageSize,
                    token = "Bearer ${SharedPreferenceManager.token}"
                )
                    .subscribeOn(Schedulers.io())
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe(
                        {
                            if (it.success && it.data != null) {
                                val data = it.data

                                _totalCountLiveData.value = data.totalCount
                                addApplicantList(data.items)

                                if (data.items.isEmpty()) {
                                    isLast = true
                                } else {
                                    page += 1
                                }
                            } else {
                                if (it.message != null) {
                                    _toastLiveData.value = it.message
                                } else {
                                    _toastLiveData.value = "알 수 없는 오류가 발생했습니다. 다시 시도해 주세요."
                                }
                            }

                            _isLoading.value = false
                        },
                        {
                            _isLoading.value = false
                            it.message?.let { message -> Logger.e(message) }
                            _toastLiveData.postValue("알 수 없는 오류가 발생했습니다. 다시 시도해 주세요.")
                        }
                    )
            )
        }
    }

    fun applyAudition(auditionRoleId: Long, phoneNumber: String, onSuccess: () -> Unit) {
        if (audioFile == null) {
            _toastLiveData.value = "잘못된 녹음 파일 입니다.\n다시 선택해 주세요."
            return
        }

        _isLoading.value = true

        val request = ApplyAuditionRoleRequest(roleId = auditionRoleId, phoneNumber = phoneNumber)
        val requestJson = Gson().toJson(request)

        val contentFile = MultipartBody.Part.createFormData(
            "contentFile",
            audioFile!!.name,
            body = object : RequestBody() {
                override fun contentType(): MediaType {
                    return "audio/*".toMediaType()
                }

                override fun writeTo(sink: BufferedSink) {
                    audioFile!!.inputStream().use { inputStream ->
                        val buffer = ByteArray(512)
                        var bytesRead: Int
                        while (inputStream.read(buffer).also { bytesRead = it } != -1) {
                            sink.write(buffer, 0, bytesRead)
                        }
                    }
                }

                override fun contentLength(): Long {
                    return audioFile!!.length()
                }
            }
        )

        compositeDisposable.add(
            repository.applyAudition(
                contentFile = contentFile,
                request = requestJson.toRequestBody("text/plain".toMediaType()),
                token = "Bearer ${SharedPreferenceManager.token}"
            ).subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(
                    {
                        _isLoading.value = false
                        if (it.success) {
                            page = 1
                            isLast = false
                            getAuditionRoleDetail(auditionRoleId = auditionRoleId)
                            onSuccess()
                        } else {
                            if (it.message != null) {
                                _toastLiveData.postValue(it.message)
                            } else {
                                _toastLiveData.postValue(
                                    "알 수 없는 오류가 발생했습니다. 다시 시도해 주세요."
                                )
                            }
                        }
                    },
                    {
                        _isLoading.value = false
                        it.message?.let { message -> Logger.e(message) }
                        _toastLiveData.postValue(
                            "알 수 없는 오류가 발생했습니다. 다시 시도해 주세요."
                        )
                    }
                )
        )
    }

    fun setSortType(sortType: AuditionApplicantSortType) {
        val prevSortType = _sortTypeLiveData.value!!

        if (sortType != prevSortType) {
            _sortTypeLiveData.value = sortType
            isLast = false
            page = 1
            getAuditionApplicantList()
        }
    }

    fun voteApplicant(position: Int, onSuccess: () -> Unit, onFailure: (() -> Unit)? = null) {
        val applicantId = _applicantListLiveData.value?.get(position)?.applicantId

        if (applicantId != null) {
            _isLoading.value = true
            val request = VoteAuditionApplicantRequest(
                applicantId = applicantId,
                timezone = TimeZone.getDefault().id
            )

            compositeDisposable.add(
                repository.voteApplicant(
                    request = request,
                    token = "Bearer ${SharedPreferenceManager.token}"
                )
                    .subscribeOn(Schedulers.io())
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe(
                        {
                            if (it.success) {
                                val updatedList =
                                    _applicantListLiveData.value?.mapIndexed { index, item ->
                                        if (index == position) {
                                            item.copy(voteCount = item.voteCount + 1)
                                        } else {
                                            item
                                        }
                                    }
                                _applicantListLiveData.value = updatedList!!
                                onSuccess()
                            } else {
                                if (it.message != null) {
                                    if (
                                        it.message.contains("오늘 응원은 여기까지") &&
                                        onFailure != null
                                    ) {
                                        onFailure()
                                    } else {
                                        _toastLiveData.value = it.message
                                    }
                                } else {
                                    _toastLiveData.value =
                                        "알 수 없는 오류가 발생했습니다. 다시 시도해 주세요."
                                }
                            }

                            _isLoading.value = false
                        },
                        {
                            _isLoading.value = false
                            it.message?.let { message -> Logger.e(message) }
                            _toastLiveData.value = "알 수 없는 오류가 발생했습니다. 다시 시도해 주세요."
                        }
                    )
            )
        }
    }

    private fun addApplicantList(itemList: List<GetAuditionRoleApplicantItem>) {
        val updatedList = if (page == 1) {
            mutableListOf()
        } else {
            _applicantListLiveData.value?.toMutableList() ?: mutableListOf()
        }

        updatedList.addAll(itemList)
        _applicantListLiveData.value = updatedList
    }

    enum class AuditionApplicantSortType {
        @SerializedName("NEWEST")
        NEWEST,

        @SerializedName("LIKES")
        LIKES
    }
}
