package kr.co.vividnext.sodalive.live.now

import android.annotation.SuppressLint
import android.content.Context
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.constraintlayout.widget.ConstraintLayout
import androidx.recyclerview.widget.RecyclerView
import coil.transform.CircleCropTransformation
import com.bumptech.glide.Glide
import com.bumptech.glide.load.resource.bitmap.CenterCrop
import com.bumptech.glide.load.resource.bitmap.RoundedCorners
import com.bumptech.glide.request.RequestOptions
import kr.co.vividnext.sodalive.R
import kr.co.vividnext.sodalive.databinding.ItemLiveNowBinding
import kr.co.vividnext.sodalive.extensions.dpToPx
import kr.co.vividnext.sodalive.extensions.loadUrl
import kr.co.vividnext.sodalive.live.GetRoomListResponse

class LiveNowAdapter(
    private val onClick: (GetRoomListResponse) -> Unit
) : RecyclerView.Adapter<LiveNowAdapter.ViewHolder>() {

    var items = mutableListOf<GetRoomListResponse>()

    inner class ViewHolder(
        private val context: Context,
        private val binding: ItemLiveNowBinding
    ) : RecyclerView.ViewHolder(binding.root) {

        @SuppressLint("SetTextI18n")
        fun bind(item: GetRoomListResponse) {
            Glide
                .with(context)
                .load(item.coverImageUrl)
                .apply(
                    RequestOptions().transform(
                        CenterCrop(),
                        RoundedCorners(8)
                    )
                )
                .into(binding.ivCover)

            val layoutParams = binding.ivCover.layoutParams as ConstraintLayout.LayoutParams
            layoutParams.width = 128f.dpToPx().toInt()
            layoutParams.height = 179f.dpToPx().toInt()
            binding.ivCover.layoutParams = layoutParams

            binding.ivLock.visibility = if (item.isPrivateRoom) {
                View.VISIBLE
            } else {
                View.GONE
            }

            if (item.price > 0) {
                binding.tvPrice.text = "${item.price}"
                binding.tvPrice.setCompoundDrawablesWithIntrinsicBounds(
                    R.drawable.ic_can_white,
                    0,
                    0,
                    0
                )
                binding.tvPrice.setBackgroundResource(R.drawable.bg_round_corner_13_3_dd4500)
            } else {
                binding.tvPrice.text = "무료"
                binding.tvPrice.setCompoundDrawables(null, null, null, null)
                binding.tvPrice.setBackgroundResource(R.drawable.bg_round_corner_13_3_111111)
            }

            if (item.tags.isNotEmpty()) {
                binding.tvTags.visibility = View.VISIBLE
                binding.tvTags.text = item.tags.joinToString(" ") { "#$it" }
            } else {
                binding.tvTags.visibility = View.GONE
            }

            binding.tvTitle.text = item.title
            binding.tvNickname.text = item.creatorNickname
            binding.ivProfile.loadUrl(item.creatorProfileImage) {
                crossfade(true)
                placeholder(R.drawable.ic_place_holder)
                transformations(CircleCropTransformation())
            }

            binding.root.setOnClickListener { onClick(item) }
        }
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int) = ViewHolder(
        parent.context,
        ItemLiveNowBinding.inflate(
            LayoutInflater.from(parent.context),
            parent,
            false
        )
    )

    override fun onBindViewHolder(holder: ViewHolder, position: Int) {
        holder.bind(items[position])
    }

    override fun getItemCount() = items.count()

    @SuppressLint("NotifyDataSetChanged")
    fun clear() {
        items.clear()
        notifyDataSetChanged()
    }
}
