package kr.co.vividnext.sodalive.message.text

import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import com.orhanobut.logger.Logger
import io.reactivex.rxjava3.android.schedulers.AndroidSchedulers
import io.reactivex.rxjava3.schedulers.Schedulers
import kr.co.vividnext.sodalive.base.BaseViewModel
import kr.co.vividnext.sodalive.common.SharedPreferenceManager
import kr.co.vividnext.sodalive.message.MessageRepository
import kr.co.vividnext.sodalive.message.SendTextMessageRequest

class TextMessageWriteViewModel(private val repository: MessageRepository) : BaseViewModel() {
    var textMessage = ""
    var recipientId: Long = 0

    private val _toastLiveData = MutableLiveData<String?>()
    val toastLiveData: LiveData<String?>
        get() = _toastLiveData

    private var _isLoading = MutableLiveData(false)
    val isLoading: LiveData<Boolean>
        get() = _isLoading

    fun write(onSuccess: () -> Unit) {
        if (recipientId <= 0) {
            _toastLiveData.postValue("받는 사람을 선택해 주세요.")
            return
        }

        if (textMessage.isBlank()) {
            _toastLiveData.postValue("메시지를 입력하세요.")
            return
        }

        val request = SendTextMessageRequest(recipientId = recipientId, textMessage = textMessage)
        _isLoading.value = true
        compositeDisposable.add(
            repository.sendTextMessage(
                request = request,
                token = "Bearer ${SharedPreferenceManager.token}"
            )
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(
                    {
                        _isLoading.value = false
                        if (it.success) {
                            onSuccess()
                        } else {
                            if (it.message != null) {
                                _toastLiveData.postValue(it.message)
                            } else {
                                _toastLiveData.postValue(
                                    "알 수 없는 오류가 발생했습니다. 다시 시도해 주세요."
                                )
                            }
                        }
                    },
                    {
                        it.message?.let { message -> Logger.e(message) }
                        _toastLiveData.postValue("알 수 없는 오류가 발생했습니다. 다시 시도해 주세요.")
                        _isLoading.postValue(false)
                    }
                )
        )
    }
}
