package kr.co.vividnext.sodalive.mypage.alarm

import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import com.orhanobut.logger.Logger
import io.reactivex.rxjava3.android.schedulers.AndroidSchedulers
import io.reactivex.rxjava3.schedulers.Schedulers
import kr.co.vividnext.sodalive.base.BaseViewModel
import kr.co.vividnext.sodalive.common.SharedPreferenceManager

class AlarmListViewModel(private val repository: AlarmListRepository) : BaseViewModel() {
    private val _toastLiveData = MutableLiveData<String?>()
    val toastLiveData: LiveData<String?>
        get() = _toastLiveData

    private var _isLoading = MutableLiveData(false)
    val isLoading: LiveData<Boolean>
        get() = _isLoading

    private var _maxAlarmSlotLiveData = MutableLiveData(3)
    val maxAlarmSlotLiveData: LiveData<Int>
        get() = _maxAlarmSlotLiveData

    private var _extraSlotPriceLiveData = MutableLiveData(0)
    val extraSlotPriceLiveData: LiveData<Int>
        get() = _extraSlotPriceLiveData

    fun getSlotQuantityAndPrice() {
        _isLoading.value = true
        compositeDisposable.add(
            repository.getSlotQuantityAndPrice("Bearer ${SharedPreferenceManager.token}")
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(
                    {
                        _isLoading.value = false
                        if (it.success && it.data != null) {
                            if (it.data.slotQuantity >= 3) {
                                _maxAlarmSlotLiveData.value = 0
                            } else {
                                _maxAlarmSlotLiveData.value = 3 + it.data.slotQuantity
                            }

                            _extraSlotPriceLiveData.value = it.data.price
                        } else {
                            if (it.message != null) {
                                _toastLiveData.postValue(it.message)
                            } else {
                                _toastLiveData.postValue(
                                    "알 수 없는 오류가 발생했습니다. 다시 시도해 주세요."
                                )
                            }
                        }
                    },
                    {
                        _isLoading.value = false
                        it.message?.let { message -> Logger.e(message) }
                        _toastLiveData.postValue("알 수 없는 오류가 발생했습니다. 다시 시도해 주세요.")
                    }
                )
        )
    }

    fun buyExtraSlot() {
        _isLoading.value = true
        compositeDisposable.add(
            repository.buyExtraSlot("Bearer ${SharedPreferenceManager.token}")
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(
                    {
                        _isLoading.value = false
                        if (it.success) {
                            getSlotQuantityAndPrice()
                        } else {
                            if (it.message != null) {
                                _toastLiveData.postValue(it.message)
                            } else {
                                _toastLiveData.postValue(
                                    "알 수 없는 오류가 발생했습니다. 다시 시도해 주세요."
                                )
                            }
                        }
                    },
                    {
                        _isLoading.value = false
                        it.message?.let { message -> Logger.e(message) }
                        _toastLiveData.postValue("알 수 없는 오류가 발생했습니다. 다시 시도해 주세요.")
                    }
                )
        )
    }
}
