package kr.co.vividnext.sodalive.settings.notice

import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import com.orhanobut.logger.Logger
import io.reactivex.rxjava3.android.schedulers.AndroidSchedulers
import io.reactivex.rxjava3.schedulers.Schedulers
import kr.co.vividnext.sodalive.base.BaseViewModel
import kr.co.vividnext.sodalive.common.SharedPreferenceManager

class NoticeViewModel(private val repository: NoticeRepository) : BaseViewModel() {
    private val _noticeLiveData = MutableLiveData<List<NoticeItem>>()
    val noticeLiveData: LiveData<List<NoticeItem>>
        get() = _noticeLiveData

    private val _toastLiveData = MutableLiveData<String?>()
    val toastLiveData: LiveData<String?>
        get() = _toastLiveData

    private var _isLoading = MutableLiveData(false)
    val isLoading: LiveData<Boolean>
        get() = _isLoading

    private var totalCount = -1
    private var page = 1
    private val size = 10

    fun getNotices() {
        if (totalCount < 0 || page * size < totalCount) {
            _isLoading.value = true
            compositeDisposable.add(
                repository.getNotices(
                    page = page - 1,
                    size = size,
                    token = "Bearer ${SharedPreferenceManager.token}"
                )
                    .subscribeOn(Schedulers.io())
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe(
                        {
                            if (it.success && it.data != null) {
                                val data = it.data
                                this.totalCount = data.totalCount
                                _noticeLiveData.postValue(data.noticeList)
                                page += 1
                            } else {
                                if (it.message != null) {
                                    _toastLiveData.postValue(it.message)
                                } else {
                                    _toastLiveData.postValue(
                                        "알 수 없는 오류가 발생했습니다. 다시 시도해 주세요."
                                    )
                                }
                            }
                            _isLoading.value = false
                        },
                        {
                            _isLoading.value = false
                            it.message?.let { message -> Logger.e(message) }
                            _toastLiveData.postValue("알 수 없는 오류가 발생했습니다. 다시 시도해 주세요.")
                        }
                    )
            )
        }
    }
}
