package kr.co.vividnext.sodalive.user.find_password

import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import com.orhanobut.logger.Logger
import io.reactivex.rxjava3.android.schedulers.AndroidSchedulers
import io.reactivex.rxjava3.schedulers.Schedulers
import kr.co.vividnext.sodalive.base.BaseViewModel
import kr.co.vividnext.sodalive.user.UserRepository

class FindPasswordViewModel(private val repository: UserRepository) : BaseViewModel() {

    var email = ""

    private var _isLoading = MutableLiveData(false)
    val isLoading: LiveData<Boolean>
        get() = _isLoading

    private val _toastLiveData = MutableLiveData<String?>()
    val toastLiveData: LiveData<String?>
        get() = _toastLiveData

    fun findPassword(onSuccess: () -> Unit) {
        if (email.isBlank()) {
            _toastLiveData.postValue("이메일을 입력하세요.")
            return
        }

        _isLoading.value = true
        val request = ForgotPasswordRequest(email = email)
        compositeDisposable.add(
            repository.findPassword(request)
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(
                    {
                        _isLoading.value = false
                        if (it.success) {
                            _toastLiveData.postValue(
                                "임시 비밀번호가 입력하신 이메일로 발송되었습니다. 이메일을 확인해 주세요."
                            )
                            onSuccess()
                        } else {
                            if (it.message != null) {
                                _toastLiveData.postValue(it.message)
                            } else {
                                _toastLiveData.postValue(
                                    "알 수 없는 오류가 발생했습니다. 다시 시도해 주세요."
                                )
                            }
                        }
                    },
                    {
                        _isLoading.value = false
                        it.message?.let { message -> Logger.e(message) }
                        _toastLiveData.postValue("알 수 없는 오류가 발생했습니다. 다시 시도해 주세요.")
                    }
                )
        )
    }
}
